
'''
Some glue code to do workspace related things based on visgraph
'''
import time
import logging
import collections

import envi
import envi.const as e_const

import visgraph.pathcore as vg_pathcore
import visgraph.graphcore as vg_graphcore

import vivisect

xrskip = envi.BR_PROC | envi.BR_DEREF

logger = logging.getLogger(__name__)


def getNodeWeightHisto(g):
    '''
    Takes a graph and returns the following tuple:
        (weights_to_node, nodes_to_weight, leaves)

    where:
        weights_to_node - dict using weight as key
        nodes_to_weight - dict using nodes as key
        leaves          - dict of nodes without refs from
    '''
    nodeweights = g.getHierNodeWeights()
    leaves = collections.defaultdict(list)
    weights_to_cb = collections.defaultdict(list)

    # create default dict
    for cb, weight in sorted(nodeweights.items(), key=lambda x: x[1]):
        if not len(g.getRefsFromByNid(cb)):
            # leaves is a tuple of (cb, current path, visited nodes)
            # these are our leaf nodes
            leaves[weight].append((cb, list(), set()))

        # create histogram
        weights_to_cb[weight].append((cb, list(), set()))

    return weights_to_cb, nodeweights, leaves

def getLongPath(g):
    '''
    Yield a list of list tuples (node id, edge id) representing the longest path
    '''

    weights_to_cb, cb_to_weights, todo = getNodeWeightHisto(g)

    # unique root node code blocks
    rootnodes = set([cb for cb, nprops in g.getHierRootNodes()])
    leafmax = 0
    if len(todo):
        leafmax = max(todo.keys())

    invalidret = False
    # if the weight of the longest path to a leaf node
    # is not the highest weight then we need to fix our
    # path choices by taking the longer path
    weightmax = max(weights_to_cb.keys())
    if leafmax != weightmax:
        todo = weights_to_cb
        leafmax = weightmax
        invalidret = True

    # this is our loop that we want to yield out of..
    # start at the bottom of the graph and work our way back up
    for weight in range(leafmax, -1, -1):
        # the todo is a a list of codeblocks a specific level
        codeblocks = todo.get(weight)
        if not codeblocks:
            continue

        for cbva, paths, visited in codeblocks:
            tleafs = collections.defaultdict(list)
            if not paths:
                paths = [(cbva, None)]
            # work is a tuple of (cbva, weight, current path, visited)
            work = [(cbva, weight, paths, visited)]
            while work:
                cbva, weight, cpath, visited = work.pop()
                upweight = weight - 1
                for eid, fromid, toid, einfo in g.getRefsToByNid(cbva):
                    if fromid in visited:
                        continue
                    nweight = cb_to_weights.get(fromid)
                    newcpath = list(cpath)
                    newcpath[-1] = (cbva, eid)
                    newcpath.append((fromid, None))
                    newvisited = set(visited)
                    newvisited.add(fromid)
                    if nweight == upweight:
                        # we've moved back one level
                        work.append((fromid, upweight, newcpath, newvisited))
                    else:
                        t = (fromid, newcpath, newvisited)
                        if t not in tleafs[nweight]:
                            tleafs[nweight].append(t)
                if cbva in rootnodes:
                    l = list(cpath)
                    l.reverse()
                    yield l

            # update our todo with our new paths to resume from
            for nw, l in tleafs.items():
                todo[nw] += l

def _nodeedge(tnode):
    nid = vg_pathcore.getNodeProp(tnode, 'nid')
    eid = vg_pathcore.getNodeProp(tnode, 'eid')
    return nid,eid

def _nodeedgeloop(tnode):
    nid = vg_pathcore.getNodeProp(tnode, 'nid')
    eid = vg_pathcore.getNodeProp(tnode, 'eid')
    loop = vg_pathcore.getNodeProp(tnode, 'loops')
    return nid,eid,loop

def getCoveragePaths(fgraph, maxpath=None):
    '''
    Get a set of paths which will cover every block, but will
    *end* on branches which re-merge with previously traversed
    paths.  This allows a full coverage of the graph with as
    little work as possible, but *will* omit possible states.

    Returns: yield based path generator ( where path is list if (nid,edge) tuples )
    '''
    pathcnt = 0
    nodedone = {}

    for root in fgraph.getHierRootNodes():

        proot = vg_pathcore.newPathNode(nid=root[0], eid=None)
        todo = [(root,proot), ]

        while todo:

            node,cpath = todo.pop()
            refsfrom = fgraph.getRefsFrom(node)

            # Record that we have visited this node...
            nodedone[node[0]] = True

            # This is a leaf node!
            if not refsfrom:
                path = vg_pathcore.getPathToNode(cpath)
                yield [ _nodeedge(n) for n in path ]

                pathcnt += 1
                if maxpath is not None and pathcnt >= maxpath:
                    return

            for eid, fromid, toid, einfo in refsfrom:

                # If we're branching to a visited node, return the path as is
                if nodedone.get(toid):
                    path = vg_pathcore.getPathToNode(cpath)
                    yield [ _nodeedge(n) for n in path ]

                    # Check if that was the last path we should yield
                    pathcnt += 1
                    if maxpath is not None and pathcnt >= maxpath:
                        return

                    # If we're at a completed node, take no further branches
                    continue

                npath = vg_pathcore.newPathNode(parent=cpath, nid=toid, eid=eid)
                tonode = fgraph.getNode(toid)
                todo.append((tonode,npath))

def getCodePathsThru(fgraph, tgtcbva, loopcnt=0, maxpath=None):
    '''
    Yields all the paths through the hierarchical graph which pass through
    the target codeblock "tgtcb".  Each "root" node is traced to the target,
    and all paths are traversed from there to the end.  Specify a loopcnt
    to allow loop paths to be generated with the given "loop iteration count"

    Example:
        for path in getCodePathsThru(fgraph, tgtcb):
            for node,edge in path:
                ...etc...
    '''
    cnt = 0
    for pathto in getCodePathsTo(fgraph, tgtcbva, loopcnt=loopcnt, maxpath=maxpath):
        for pathfrom in getCodePathsFrom(fgraph, tgtcbva, loopcnt=loopcnt, maxpath=maxpath):
            yield pathto + pathfrom[1:]
            cnt += 1
            if maxpath is not None and cnt >= maxpath:
                return

def getCodePathsTo(fgraph, tocbva, loopcnt=0, maxpath=None):
    '''
    Yields all the paths through the hierarchical graph starting at the
    "root nodes" and ending at tocbva.  Specify a loopcnt to allow loop
    paths to be generated with the given "loop iteration count"

    Example:
        for path in getCodePathsTo(fgraph, tocbva):
            for node,edge in path:
                ...etc...
    '''
    pathcnt = 0
    looptrack = []
    pnode = vg_pathcore.newPathNode(nid=tocbva, eid=None)

    node = fgraph.getNode(tocbva)
    todo = [(node,pnode), ]

    while todo:

        node,cpath = todo.pop()

        refsto = fgraph.getRefsTo(node)

        # Is this is the root node?
        if node[1].get('rootnode'):
            path = vg_pathcore.getPathToNode(cpath)
            path.reverse()
            yield [ _nodeedge(n) for n in path ]
            vg_pathcore.trimPath(cpath)

            pathcnt += 1
            if maxpath and pathcnt >= maxpath:
                return

        for eid, n1, n2, einfo in refsto:
            # Skip loops if they are "deeper" than we are allowed
            loops = vg_pathcore.getPathLoopCount(cpath, 'nid', n1)
            if loops > loopcnt:
                continue

            vg_pathcore.setNodeProp(cpath, 'eid', eid)
            npath = vg_pathcore.newPathNode(parent=cpath, nid=n1, eid=None)
            nid1,node1 = fgraph.getNode(n1)
            todo.append(((nid1,node1),npath))

def getCodePathsFrom(fgraph, fromcbva, loopcnt=0, maxpath=None):
    '''
    Yields all the paths through the hierarchical graph beginning with 
    "fromcbva", which is traced to all terminating points.  Specify a loopcnt
    to allow loop paths to be generated with the given "loop iteration count"

    Example:
        for path in getCodePathsFrom(fgraph, fromcbva):
            for node,edge in path:
                ...etc...
    '''
    pathcnt = 0
    proot = vg_pathcore.newPathNode(nid=fromcbva, eid=None)

    cbnid,cbnode = fgraph.getNode(fromcbva)
    todo = [(cbnid,proot), ]

    while todo:

        nid,cpath = todo.pop()

        refsfrom = fgraph.getRefsFromByNid(nid)

        # This is a leaf node!
        if not refsfrom:
            path = vg_pathcore.getPathToNode(cpath)
            yield [ _nodeedge(n) for n in path ]
            vg_pathcore.trimPath(cpath)

            pathcnt += 1
            if maxpath and pathcnt >= maxpath:
                return

        for eid, fromid, n2, einfo in refsfrom:
            # Skip loops if they are "deeper" than we are allowed
            loops = vg_pathcore.getPathLoopCount(cpath, 'nid', n2)
            if loops > loopcnt:
                continue

            npath = vg_pathcore.newPathNode(parent=cpath, nid=n2, eid=eid)
            todo.append((n2,npath))

def getCodePaths(fgraph, loopcnt=0, maxpath=None):
    '''
    Yields all the paths through the hierarchical graph.  Each
    "root" node is traced to all terminating points.  Specify a loopcnt
    to allow loop paths to be generated with the given "loop iteration count"

    Example:
        for path in getCodePaths(fgraph):
            for node,edge in path:
                ...etc...
    '''
    pathcnt = 0
    for root in fgraph.getHierRootNodes():
        proot = vg_pathcore.newPathNode(nid=root[0], eid=None)
        todo = [(root,proot), ]

        while todo:

            node,cpath = todo.pop()

            refsfrom = fgraph.getRefsFrom(node)

            # This is a leaf node!
            if not refsfrom:
                path = vg_pathcore.getPathToNode(cpath)
                yield [ _nodeedge(n) for n in path ]
                vg_pathcore.trimPath(cpath)

                pathcnt += 1
                if maxpath and pathcnt >= maxpath:
                    return

            for eid, fromid, toid, einfo in refsfrom:
                # Skip loops if they are "deeper" than we are allowed
                if vg_pathcore.getPathLoopCount(cpath, 'nid', toid) > loopcnt:
                    continue

                npath = vg_pathcore.newPathNode(parent=cpath, nid=toid, eid=eid)
                tonode = fgraph.getNode(toid)
                todo.append((tonode,npath))

def walkCodePaths(fgraph, callback, loopcnt=0, maxpath=None):
    '''
    walkCodePaths is a path generator which uses a callback function to determine the 
    viability of each particular path.  This approach allows the calling function 
    (eg. walkSymbolikPaths) to do in-generator checks/processing and trim paths which
    are simply not possible/desireable.

    Callbacks will receive the current path, the current edge, and the new path node.
    For root nodes, the current path and edge will be None types.  
    '''
    pathcnt = 0
    routed = fgraph.getMeta('Routed', False)
    for root in fgraph.getHierRootNodes():
        proot = vg_pathcore.newPathNode(nid=root[0], eid=None)

        # Fire callback once to init the dest "path node"
        callback(None, None, proot)

        todo = [(root,proot), ]

        while todo:

            node,cpath = todo.pop()
            refsfrom = fgraph.getRefsFrom(node)

            # This is a leaf node!
            if not refsfrom:
                #path = vg_pathcore.getPathToNode(cpath)
                #yield [ _nodeedge(n) for n in path ]

                # let the callback know we've reached one...
                #if callback(cpath, None, None):
                yield cpath

                vg_pathcore.trimPath(cpath)

                pathcnt += 1
                if maxpath and pathcnt >= maxpath:
                    return

            for eid, fromid, toid, einfo in refsfrom:
                # skip edges which are not marked "follow"
                if routed and not einfo.get('follow', False):
                    continue
                # Skip loops if they are "deeper" than we are allowed
                if vg_pathcore.getPathLoopCount(cpath, 'nid', toid) > loopcnt:
                    continue

                edge = (eid,fromid,toid,einfo)

                npath = vg_pathcore.newPathNode(parent=cpath, nid=toid, eid=eid)

                if not callback(cpath, edge, npath):
                    vg_pathcore.trimPath(npath)
                    continue

                todo.append((fgraph.getNode(toid),npath))

def getLoopPaths(fgraph):
    '''
    Similar to getCodePaths(), however, getLoopPaths() will return path lists
    which loop.  The last element in the (node,edge) list will be the first
    "looped" block.
    '''
    for root in fgraph.getHierRootNodes():
        proot = vg_pathcore.newPathNode(nid=root[0], eid=None)
        todo = [ (root[0],proot,0), ]

        while todo:
            node,cpath,loopcnt = todo.pop()

            count = 0
            free = []
            if loopcnt == 1:
                yield [ _nodeedge(n) for n in vg_pathcore.getPathToNode(npath) ]

            else:
                for eid, fromid, toid, einfo in fgraph.getRefsFromByNid(node):

                    loopcnt = vg_pathcore.getPathLoopCount(cpath, 'nid', toid)
                    if loopcnt > 1:
                        continue

                    count += 1
                    npath = vg_pathcore.newPathNode(parent=cpath, nid=toid, eid=eid)
                    todo.append((toid,npath,loopcnt))

            if not count:
                vg_pathcore.trimPath(cpath)

def getOpsFromPath(vw, fgraph, path):
    '''
    Retrieve the opcodes for a given path.

    '''
    # FIXME cache opcodes in function graph for replay speed
    ret = []
    for nid,eid in path:
        node = fgraph.getNode(nid)
        cbva = node[1].get('cbva')
        cbmax = cbva + node[1].get('cbsize')
        while cbva < cbmax:
            op = vw.parseOpcode(cbva)
            ret.append(op)
            cbva += op.size
    return ret

def buildFunctionGraph(vw, fva, revloop=False, g=None):
    '''
    Build a visgraph HierGraph for the specified function.
    '''

    if g is None:
        g = vg_graphcore.HierGraph()
        g.setMeta('fva', fva)

    colors = vw.getFunctionMeta(fva, 'BlockColors', default={})
    fcb = vw.getCodeBlock(fva)
    if fcb is None:
        t = (fva, vw.isFunction(fva))
        raise Exception('Invalid initial code block for 0x%.8x isfunc: %s' % t)

    todo = [ (fcb, []), ]

    fcbva, fcbsize, fcbfunc = fcb

    # Add the root node...
    bcolor = colors.get(fva, '#0f0')
    g.addNode(nid=fva, rootnode=True, cbva=fva, cbsize=fcbsize, color=bcolor)

    while todo:

        (cbva, cbsize, cbfunc), path = todo.pop()

        path.append(cbva)

        # If the code block va doesn't have a node yet, make one
        if not g.hasNode(cbva):
            bcolor = colors.get(cbva, '#0f0')
            g.addNode(nid=cbva, cbva=cbva, cbsize=cbsize, color=bcolor)

        # Grab the location for the last instruction in the block
        nextva = cbva + cbsize - 1
        loc = vw.getLocation(nextva)
        if loc is None:
            raise Exception("buildFunctionGraph: Attempt to get location at 0x%x" % nextva)

        lva, lsize, ltype, linfo = loc

        for xrfrom, xrto, xrtype, xrflags in vw.getXrefsFrom(lva, vivisect.REF_CODE):

            # For now, the graph doesn't cross function boundaries
            # or indirects.
            if xrflags & xrskip:
                continue

            mmap = vw.getMemoryMap(xrto)
            if mmap:
                mva, msize, mperm, mname = mmap
                if mperm & e_const.MM_UNINIT:
                    continue

            if not g.hasNode(xrto):
                cblock = vw.getCodeBlock(xrto)
                if cblock is None:
                    logger.warning('CB is None in graph building?!?! (0x%x)', xrto)
                    logger.warning('(fva: 0x%.8x cbva: 0x%.8x)', fva, xrto)
                    continue

                tova, tosize, tofunc = cblock
                if tova != xrto:
                    logger.warning('CBVA != XREFTO in graph building!?')
                    logger.warning('(cbva: 0x%.8x xrto: 0x%.8x)', tova, xrto)
                    continue

                # Since we haven't seen this node, lets add it to todo
                # and build a new node for it.
                todo.append(((tova,tosize,tofunc), list(path)))
                bcolor = colors.get(tova, '#0f0')
                g.addNode(nid=tova, cbva=tova, cbsize=tosize, color=bcolor)

            # If they want it, reverse "loop" edges (graph layout...)
            if revloop and xrto in path:
                g.addEdgeByNids(xrto, cbva, reverse=True)
            else:
                g.addEdgeByNids(cbva, xrto)

        if ltype == vivisect.LOC_OP and linfo & envi.IF_NOFALL:
            continue

        # If this codeblock can fall through into another, add it to
        # todo!
        fallva = lva + lsize
        if not g.hasNode(fallva):
            fallblock = vw.getCodeBlock(fallva)
            if fallblock is None:
                logger.warning('FB is None in graph building!??!')
                logger.warning('(fva: 0x%.8x  fallva: 0x%.8x', fva, fallva)
            elif fallva != fallblock[0]:
                logger.warning('FALLVA != CBVA in graph building!??!')
                logger.warning('(fallva: 0x%.8x CBVA: 0x%.8x', fallva, fallblock[0])
            else:
                fbva, fbsize, fbfunc = fallblock
                #if fbfunc != fva and fbva not in blocks:
                #    continue

                todo.append( ((fbva,fbsize,fbfunc), list(path)) )
                bcolor = colors.get(fallva, '#0f0')
                g.addNode(nid=fallva, cbva=fallva, cbsize=fbsize, color=bcolor)

        # If we ended up with a destination node, make the edge
        if g.hasNode(fallva):
            # If they want it, reverse "loop" edges (graph layout...)
            if revloop and fallva in path:
                g.addEdgeByNids(fallva, cbva, reverse=True)
            else:
                g.addEdgeByNids(cbva, fallva)

    return g

def getGraphNodeByVa(fgraph, va):
    '''
    Returns graph node a given VA falls within.
    Similar to VivWorkspace.getCodeBlock(va).

    Because this involves the concept of CodeBlocks, it does not fit in the
    GraphCore.

    DEPRECATED as soon as visi's new CodeGraph gains this functionality inherently
    '''
    for nva, ninfo in fgraph.nodes.values():
        nvamax = ninfo.get('cbsize')
        if nvamax is None: 
            raise Exception('getGraphNodeByVa() called on graph with non-codeblock nodes')

        nvamax += nva
        if va >= nva and va < nvamax:
            return nva
    return None

def findRemergeDown(graph, va):
    '''
    starting at a given va, figure out the nodes connecting va to the next place something remerges
    '''
    startnid = getGraphNodeByVa(graph, va)

    # paint down graph, 
    preRouteGraphDown(graph, startnid, mark='hit', loop=False)

    histo, nodewts, leaves = getNodeWeightHisto(graph)
    startnode = graph.getNode(startnid)
    startweight = nodewts.get(startnid)

    for node in graph.getNodesByProp('hit'):
        # skip the starting node
        if node[0] == startnid: 
            continue

        if node[1].get('hit') is None: 
            continue

        for eid, frva, tova, einfo in graph.getRefsTo(node):
            frnode = graph.getNode(frva)
            if frnode[1].get('hit') is None:
                # clear from here down
                clearMarkDown(graph, tova, mark='hit')
                break

# path routing through a graph.  reduces aimless wandering when we know where we want to be
def preRouteGraph(graph, fromva, tova, clearFirst=True):
    '''
    Package it all together
    '''
    if clearFirst:
        clearRouting(graph)

    preRouteGraphUp(graph, tova)
    preRouteGraphDown(graph, fromva)
    preRouteGraphEdges(graph)

def preRouteGraphEdges(graph):
    '''
    Mark edges as 'follow' if from-node is marked 'up' and to-node id marked 'down'
    Note: unlike the other preRoute functions, this is not flexible on naming.
    '''
    for edge in graph.getEdges():
        eid, frnid, tonid, einfo = edge
        if not graph.getNodeProps(frnid).get('up'):
            continue
        if not graph.getNodeProps(tonid).get('down'):
            continue

        graph.setEdgeProp(edge, 'follow', True)

def preRouteGraphUp(graph, tova, loop=True, mark='down'):
    '''
    paint a route from our destination, 'up' the graph
    '''
    graph.setMeta('Routed', True)
    tonid = getGraphNodeByVa(graph, tova)
    if tonid is None:
        raise Exception("tova not in graph 0x%x" % tova)

    tonode = graph.getNode(tonid)
    nwlist = graph.getHierNodeWeights()
    todo = [ (tonode) ]
    while todo:
        curnode = todo.pop()
        graph.setNodeProp(curnode, mark, True)
        for eid, fr, to, einfo in graph.getRefsTo(curnode):
            if graph.getNodeProps(fr).get(mark) == True:
                continue
            if not loop and nwlist.get(fr) <= nwlist.get(to):
                continue
            frnode = graph.getNode(fr)
            todo.append(frnode)
   
def preRouteGraphDown(graph, fromva, loop=False, mark='up'):
    '''
    paint a route from our starting point, 'down' the graph
    '''
    graph.setMeta('Routed', True)
    fromnode = getGraphNodeByVa(graph, fromva)
    if fromnode is None:
        raise Exception("fromva not in graph 0x%x" % fromva)

    nwlist = graph.getHierNodeWeights()
    todo = [ graph.getNode(fromnode) ]
    while todo:
        curnode = todo.pop()
        curnodeva, curninfo = curnode
        graph.setNodeProp(curnode, mark, True)
        for eid, fr, to, einfo in graph.getRefsFrom(curnode):
            if graph.getNodeProps(to).get(mark) == True:
                continue
            if not loop and nwlist.get(fr) >= nwlist.get(to):
                continue

            todo.append(graph.getNode(to))

def clearMarkDown(graph, fromva, loop=False, mark='up'):
    '''
    clear a route from our starting point, 'down' the graph.
    ie. remove the mark
    '''
    fromnode = getGraphNodeByVa(graph, fromva)
    if fromnode is None:
        raise Exception("fromva not in graph 0x%x" % fromva)

    nwlist = graph.getHierNodeWeights()
    todo = [ graph.getNode(fromnode) ]
    while len(todo):
        curnode = todo.pop()
        curnodeva, curninfo = curnode

        # actually delete the node property, not just remove value
        graph.delNodeProp(curnode, mark)

        for eid, fr, to, einfo in graph.getRefsFrom(curnode):
            if graph.getNodeProps(to).get(mark) == True:
                continue
            if not loop and nwlist.get(fr) >= nwlist.get(to):
                continue

            todo.append(graph.getNode(to))

def clearRouting(graph, nmarks=('up','down'), emarks=('follow',)):
    graph.delNodesProps(nmarks)
    graph.delEdgesProps(emarks)
    graph.setMeta('Routed', False)

def reduceGraph(graph, props=('up','down')):
    '''
    trims all nodes that don't have all the props in the props list
    '''
    for node in graph.getNodes():
        for prop in props:
            if node[1].get(prop) is None:
                graph.delNode(node)
                break


# TODO: Move into base exception file
class PathForceQuitException(Exception):
    def __init__(self, pathcount=None):
        Exception.__init__(self)
        self.pathcount = pathcount

    def __repr__(self):
        if self.pathcount is None:
            return "Path Generator forced to stop seeking a new path.  Possibly Timeout."

        return "Path Generator forced to stop seeking a new path.  Possibly Timeout. (paths processed: %d)" % self.pathcount

'''
eventually, routing will include the ability to 'source-route', picking N specific points a path must go through
'''
class PathGenerator:
    '''
    PathGenerator provides routed paths using yield generators, with some external 
    control.  Because these generators are typically layered with other API's 
    (ie. Symboliks subsystem calls) on top, PathGenerator provides a timeout and 
    some external control.

    PathGenerator should be used one per thread, not shared between threads.  The stop() 
    method is good for use by a single management thread.
    '''

    def __init__(self, graph):
        self.graph = graph
        self.__go__ = False

    def stop(self):
        '''
        stops path generation.
        '''
        self.__go__ = False

    def getFuncCbRoutedPaths_genback(self, fromva, tova, loopcnt=0, maxpath=None, timeout=None):
        '''
        Yields all the paths through the hierarchical graph starting at the 
        "root nodes" and ending at tocbva.  Specify a loopcnt to allow loop 
        paths to be generated with the given "loop iteration count"

        Example:
            for path in getCodePathsTo(fgraph, tocbva):
                for node,edge in path:
                    ...etc...
        '''
        fgraph = self.graph
        self.__update = 0
        self.__go__ = True
        pathcnt = 0
        tocbva = getGraphNodeByVa(fgraph, tova)
        frcbva = getGraphNodeByVa(fgraph, fromva)

        preRouteGraph(fgraph, fromva, tova)
        
        pnode = vg_pathcore.newPathNode(nid=tocbva, eid=None)

        todo = [(tocbva,pnode), ]

        maxtime = None
        if timeout:
            maxtime = time.time() + timeout

        while todo:
            if maxtime and time.time() > maxtime:
                raise PathForceQuitException(pathcnt)

            if not self.__go__:
                raise PathForceQuitException(pathcnt)

            nodeid,cpath = todo.pop()

            refsto = fgraph.getRefsTo((nodeid, None))

            # This is the root node!
            if nodeid == frcbva:
                path = vg_pathcore.getPathToNode(cpath)
                path.reverse()
                self.__steplock.acquire()
                yield [ viv_graph._nodeedge(n) for n in path ]
                vg_pathcore.trimPath(cpath)

                pathcnt += 1
                self.__update = 1
                self.__steplock.release()
                if maxpath and pathcnt >= maxpath:
                    return

            for eid, fromid, toid, einfo in refsto:
                if fgraph.getNodeProps(fromid).get('up') != True:
                    # TODO: drop the bad edges from graph in preprocessing? instead of "if" here
                    vg_pathcore.trimPath(cpath)
                    continue

                # Skip loops if they are "deeper" than we are allowed
                loops = vg_pathcore.getPathLoopCount(cpath, 'nid', fromid)
                if loops > loopcnt:
                    continue

                vg_pathcore.setNodeProp(cpath, 'eid', eid)
                npath = vg_pathcore.newPathNode(parent=cpath, nid=fromid, eid=None)
                todo.append((fromid,npath))

        self.__go__ = False

    def getFuncCbRoutedPaths(self, fromva, tova, loopcnt=0, maxpath=None, timeout=None):
        '''
        Yields all the paths through the hierarchical graph starting at the 
        "root nodes" and ending at tocbva.  Specify a loopcnt to allow loop 
        paths to be generated with the given "loop iteration count"

        Example:
            for path in getCodePathsTo(fgraph, tocbva):
                for node,edge in path:
                    ...etc...
        '''
        fgraph = self.graph
        self.__update = 0
        self.__go__ = True
        pathcnt = 0
        tocbva = getGraphNodeByVa(fgraph, tova)
        frcbva = getGraphNodeByVa(fgraph, fromva)

        # preroute
        preRouteGraph(fgraph, fromva, tova)
        reduceGraph(fgraph)
        clearRouting(fgraph)

        # get node weights for reference later
        weights = fgraph.getHierNodeWeights()
        
        pnode = vg_pathcore.newPathNode(nid=frcbva, eid=None)
        
        loopcount = 0

        todo = [(frcbva,pnode,loopcount), ]

        maxtime = None
        if timeout:
            maxtime = time.time() + timeout

        while todo:
            if maxtime and time.time() > maxtime:
                raise PathForceQuitException(pathcnt)

            if not self.__go__:
                raise PathForceQuitException(pathcnt)

            nodeid,cpath,loopcount = todo.pop()

            # This is the root node!
            if nodeid == tocbva:
                path = vg_pathcore.getPathToNode(cpath)
                yield [ _nodeedge(n) for n in path ]
                vg_pathcore.trimPath(cpath)

                pathcnt += 1
                self.__update = 1
                if maxpath and pathcnt >= maxpath:
                    return

            refsfrom = fgraph.getRefsFrom((nodeid, None))

            curlvl = weights.get(nodeid)
            for eid, fromid, toid, einfo in refsfrom:
                tgtlvl = weights.get(toid)
                if tgtlvl <= curlvl:
                    # if we're moving *up* in the world, increment and check loopcount
                    #loops = vg_pathcore.getPathLoopCount(cpath, 'nid', fromid)
                    loopcount += 1
                    if loopcount > loopcnt:
                        vg_pathcore.trimPath(cpath)
                        #sys.stderr.write('o')

                        # as long as we have at least one path, we count loops as paths, lest we die.
                        if pathcnt: 
                            pathcnt += 1
                        continue

                npath = vg_pathcore.newPathNode(parent=cpath, nid=toid, eid=eid)
                todo.append((toid,npath,loopcount))

            vg_pathcore.trimPath(cpath)

        self.__go__ = False

def pathGenConvert(pathgen):
    '''
    this takes in a HierGraph path-generator and converts each path to the kind needed by Symboliks
    '''
    for path in pathgen:
        newpath = []
        prevedge = [ None ]
        for node, nextedge in path:
            newpath.append((node[0], prevedge[0]))
            prevedge = nextedge

        #pprint(newpath)
        yield newpath




import unittest

import envi.memory as e_mem

import vivisect as viv
import vivisect.const as viv_const
import vivisect.symboliks.common as vsc
import vivisect.symboliks.archind as vsym_archind
import vivisect.symboliks.analysis as vsym_analysis


class TestArchind(unittest.TestCase):

    def test_wipeAstArch(self):
        vw = viv.VivWorkspace()
        vw.addMemoryMap(0x00, e_mem.MM_RWX, 'code', [0 for x in range(256)])
        vw.addLocation(0x40, 47, viv_const.LOC_POINTER)
        vw.setMeta('Architecture', 'i386')
        symctx = vsym_analysis.SymbolikAnalysisContext(vw)

        func = vsc.Var('eax', 4)
        call = vsc.Call(func, 4)
        addr = vsc.Var('arg0', 4) + call
        mem = vsc.Mem(addr, vsc.Const(4, 4))

        op = mem + vsc.Var('edx', 4)
        final = op * vsc.Var('edx', 4)
        wiped = vsym_archind.wipeAstArch(symctx, [final])
        self.assertEqual(1, len(wiped))
        self.assertEqual('((mem[(arg0 + 1indreg()):4] + 0indreg) * 0indreg)', str(wiped[0]))

    def test_wipeAstArch_wipeva(self):
        vw = viv.VivWorkspace()
        vw.addMemoryMap(0x410000, e_mem.MM_RWX, 'code', [0 for x in range(0x10000)])
        vw.addLocation(0x41b2ac, 47, viv_const.LOC_POINTER)
        vw.addLocation(0x4149b3, 28, viv_const.LOC_POINTER)
        vw.addLocation(0x41ac93, 83, viv_const.LOC_POINTER)
        vw.setMeta('Architecture', 'i386')
        symctx = vsym_analysis.SymbolikAnalysisContext(vw)

        cons = vsc.Const(0x41b2ac, 4)
        func1 = vsc.Call(cons, 4, argsyms=[vsc.Var('edx', 4)])
        func2 = vsc.Call(vsc.Const(0x4149b3, 4), 4, argsyms=[
                         vsc.Const(0, 4), vsc.Const(1, 4), vsc.Const(0x41b2ac, 4)])
        func3 = vsc.Call(vsc.Const(0x41ac93, 4), 4, argsyms=[
                         vsc.Const(0, 4), vsc.Var('ecx', 4)])
        wiped = vsym_archind.wipeAstArch(symctx, [func1 + func2, func3], wipeva=True)
        self.assertEqual(2, len(wiped))
        self.assertEqual('(2archindva(1indreg) + 1archindva(0,1,2archindva))', str(wiped[0]))
        self.assertEqual('0archindva(0,0indreg)', str(wiped[1]))

    def test_basic(self):
        vw = viv.VivWorkspace()
        vw.addMemoryMap(0x56560000, 7, 'woot', b'A'*8192)
        vw.setMeta('Architecture', 'i386')
        symctx = vsym_analysis.getSymbolikAnalysisContext(vw)

        base = vsc.Mem(vsc.Const(0x56560020,4),vsc.Const(4,4)) - (vsc.Var('arg0',4) + vsc.Var('ecx',4) ) * ( vsc.Var('ebx',4) + vsc.Var('ebx',4) ) + vsc.Const(9999,4)
        eqs = [
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ebx', 4)) * (vsc.Var('ecx', 4) + vsc.Var('ecx', 4)) + vsc.Const(9999, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('eax', 4) + vsc.Var('eax', 4)) * (vsc.Var('edx', 4) + vsc.Var('arg0', 4)) + vsc.Const(9999, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ecx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ebx', 4)) + vsc.Const(9999, 4),
            vsc.Mem(vsc.Const(0x56560056, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ecx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ebx', 4)) + vsc.Const(9999, 4),
        ]

        nes = [
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ebx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ecx', 4)) + vsc.Const(9999, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('eax', 4)  + vsc.Var('arg1', 4)) *(vsc.Var('edx', 4) + vsc.Var('arg0', 4))+ vsc.Const(9999, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ecx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ebx', 4)) + vsc.Const(8888, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ecx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ebx', 4)) + vsc.Const(0x56560020, 4),
            vsc.Mem(vsc.Const(0x56560020, 4), vsc.Const(4, 4)) - (vsc.Var('arg0', 4) + vsc.Var('ecx', 4)) * (vsc.Var('ebx', 4) + vsc.Var('ebx', 4)) + vsc.Const(0x56560040, 4),
        ]

        post_eqs = [
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 0x0000270f)',
            '((mem[0archindva:4] - ((1indreg + 1indreg) * (0indreg + arg0))) + 0x0000270f)',
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 0x0000270f)',
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 0x0000270f)',
        ]

        post_nes = [
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (0indreg + 1indreg))) + 0x0000270f)',
            '((mem[0archindva:4] - ((1indreg + arg1) * (0indreg + arg0))) + 0x0000270f)',
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 0x000022b8)',
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 0archindva)',
            '((mem[0archindva:4] - ((arg0 + 0indreg) * (1indreg + 1indreg))) + 1archindva)',
        ]

        for i in range(len(post_eqs)):
            eq = vsym_archind.wipeAstArch(symctx, [eqs[i]], wipeva=True)[0]
            self.assertEqual(post_eqs[i], str(eq))

        for i in range(len(post_nes)):
            ne = vsym_archind.wipeAstArch(symctx, [nes[i]], wipeva=True)[0]
            self.assertEqual(post_nes[i], str(ne))

        base = vsym_archind.wipeAstArch(symctx, [base, ], wipeva=True)[0]
        baseval = base.solve(emu=None)
        for match in eqs:
            self.assertEqual(match.solve(emu=None), baseval)

        for nonmatch in nes:
            self.assertNotEqual(nonmatch.solve(emu=None), baseval)



'''
The stalker subsystem is a breakpoint based coverage tool
'''

import vtrace

import envi
import envi.const as e_const
import envi.codeflow as e_codeflow


class StalkerCodeFlow(e_codeflow.CodeFlowContext):

    def __init__(self, trace):
        e_codeflow.CodeFlowContext.__init__(self, trace, persist=True, recurse=False)
        self.trace = trace
        self.setupBreakLists(None)

    def setupBreakLists(self, mmap):
        self.mmap = mmap
        self.bplist = []   # Block Breaks
        self.sbreaks = []  # Stalker Breaks
        self.scbreaks = []  # Callbreaks

    def _cb_opcode(self, va, op, branches):

        ret = []
        for br, bflags in branches:

            if bflags & envi.BR_DEREF and br is not None:
                bflags &= ~envi.BR_DEREF # Mask it back out...
                if not self.trace.probeMemory(br, 1, e_const.MM_READ):
                    continue

                br = self.trace.readMemoryFormat(br, '<P')[0]

            # Skip branches to other maps...
            if br is not None and self.trace.getMemoryMap(br) != self.mmap:
                continue

            ret.append((br, bflags))

            # Procedural branches to regs etc must be marked
            # Otherwise, add another breakpoint like us
            if bflags & envi.BR_PROC:
                if br is None:
                    self.scbreaks.append(op.va)
                else:
                    self.sbreaks.append(br)
                continue

            if br is None:
                self.scbreaks.append(op.va)
                continue

            # Conditional branches always create new blocks...
            if bflags & envi.BR_COND:
                self.bplist.append(br)
                continue

            # Even non-conditional jmp's will create new blocks for now...
            if br != op.va + len(op):
                self.bplist.append(br)
                continue

        return ret


class StalkerBreak(vtrace.Breakpoint):

    '''
    Stalker breakpoints are added to function entry points
    to trigger code-flow analysis and subsequent block breakpoint
    addition.
    '''

    def __init__(self, address, expression=None):
        vtrace.Breakpoint.__init__(self, address, expression=expression)
        self.fastbreak = True
        self.mymap = None

    def resolvedaddr(self, trace, address):
        vtrace.Breakpoint.resolvedaddr(self, trace, address)
        self.mymap = trace.getMemoryMap(address)

    def notify(self, event, trace):
        self.trace = trace

        # Get out of the way
        self.enabled = False

        breaks = trace.getMeta('StalkerBreaks')
        h = trace.getMeta('StalkerHits')
        h.append(self.address)

        cf = trace.getMeta('StalkerCodeFlow')
        if cf is None:
            cf = StalkerCodeFlow(trace)
            trace.setMeta('StalkerCodeFlow', cf)

        cf.setupBreakLists(self.mymap)
        cf.addCodeFlow(self.address)

        for va in cf.bplist:
            if breaks.get(va):
                continue
            breaks[va] = True
            b = StalkerBlockBreak(va)
            trace.addBreakpoint(b)

        for va in cf.sbreaks:
            if breaks.get(va):
                continue
            breaks[va] = True
            b = StalkerBreak(va)
            trace.addBreakpoint(b)

        for va in cf.scbreaks:
            if breaks.get(va):
                continue
            breaks[va] = True
            b = StalkerDynBreak(va)
            trace.addBreakpoint(b)


class StalkerBlockBreak(vtrace.Breakpoint):
    '''
    A breakpoint object which is put on codeblock boundaries
    to track hits.
    '''

    def __init__(self, address, expression=None):
        vtrace.Breakpoint.__init__(self, address, expression=expression)
        self.fastbreak = True

    def notify(self, event, trace):
        h = trace.getMeta('StalkerHits')
        h.append(self.address)
        self.enabled = False
        trace.runAgain()


class StalkerDynBreak(vtrace.Breakpoint):

    '''
    A breakpoint which is placed on dynamic branches to track
    code flow across them.
    '''

    def __init__(self, address, expression=None):
        vtrace.Breakpoint.__init__(self, address, expression=expression)
        self.fastbreak = True
        self.mymap = None
        self.lasthit = None
        self.lastcnt = 0

    def resolvedaddr(self, trace, address):
        vtrace.Breakpoint.resolvedaddr(self, trace, address)
        self.mymap = trace.getMemoryMap(address)

    def notify(self, event, trace):

        trace.runAgain()

        op = trace.parseOpcode(self.address)
        # Where is the call going?
        dva = op.getOperValue(0, emu=trace)

        if self.lasthit == dva:
            self.lastcnt += 1
        else:
            self.lasthit = dva
            self.lastcnt = 0

        if trace.getMemoryMap(dva) == self.mymap:
            addStalkerEntry(trace, dva)

        if self.lastcnt > 10:  # FIXME what should this be??!?!
            self.lasthit = None
            self.lastcnt = 0
            self.enabled = False


def initStalker(trace):
    if trace.getMeta('StalkerBreaks') is None:
        trace.setMeta('StalkerBreaks', {})
        trace.setMeta('StalkerHits', [])


def clearStalkerHits(trace):
    '''
    Clear the stalker hit list for the given trace
    '''
    initStalker(trace)
    trace.setMeta('StalkerHits', [])


def getStalkerHits(trace):
    '''
    Retrieve the list of blocks hit in the current stalker
    '''
    initStalker(trace)
    return trace.getMeta('StalkerHits', [])


def clearStalkerBreaks(trace):
    '''
    Cleanup all stalker breaks and metadata
    '''
    initStalker(trace)
    breaks = trace.getMeta('StalkerBreaks', {})
    trace.setMeta('StalkerCodeFlow', None)
    bpaddrs = list(breaks.keys())
    for va in bpaddrs:
        bp = trace.getBreakpointByAddr(va)
        if bp is not None:
            trace.removeBreakpoint(bp.id)
        breaks.pop(va, None)


def resetStalkerBreaks(trace):
    '''
    Re-enable all previously hit stalker breakpoints.
    '''
    initStalker(trace)
    breaks = trace.getMeta('StalkerBreaks', {})
    bpaddrs = list(breaks.keys())
    trace.fb_bp_done = False  # FIXME HACK
    for va in bpaddrs:
        bp = trace.getBreakpointByAddr(va)
        if bp is not None:
            trace.setBreakpointEnabled(bp.id, enabled=True)


def addStalkerEntry(trace, va):
    '''
    Add stalker coverage beginning with the specified entry point
    '''
    initStalker(trace)
    b = trace.getMeta('StalkerBreaks')
    if b.get(va):
        return
    bp = StalkerBreak(va)
    trace.addBreakpoint(bp)
    b[va] = True




"""
The analysis package.  Modules in this directory are responsible
for different phases of analysis on different platforms.
"""

import logging
logger = logging.getLogger(__name__)

ARM_ARCHS = ('arm', 'thumb', 'thumb16')


def addAnalysisModules(vw):

    import vivisect
    import vivisect.analysis.i386 as viv_analysis_i386

    arch = vw.getMeta('Architecture')
    fmt = vw.getMeta('Format')

    if fmt == 'pe':

        vw.addAnalysisModule("vivisect.analysis.generic.linker")
        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        vw.addAnalysisModule("vivisect.analysis.pe")

        if arch == 'i386':

            vw.addImpApi('windows', 'i386')
            vw.addImpApi('winkern', 'i386')

            viv_analysis_i386.addEntrySigs(vw)
            vw.addStructureModule('win32', 'vstruct.defs.win32')
            vw.addStructureModule('ntdll', 'vstruct.defs.windows.win_5_1_i386.ntdll')

        elif arch == 'amd64':

            vw.addImpApi('windows', 'amd64')
            vw.addImpApi('winkern', 'amd64')
            vw.addStructureModule('ntdll', 'vstruct.defs.windows.win_6_1_amd64.ntdll')

        elif arch in ARM_ARCHS:
            vw.addImpApi('windows','arm')
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        vw.addConstModule('vstruct.constants.ntstatus')

        vw.addAnalysisModule("vivisect.analysis.generic.relocations")

        vw.addAnalysisModule("vivisect.analysis.ms.vftables")  # RELIES ON LOC_POINTER
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")  # RELIES ON LOC_POINTER

        # run imports after emucode
        if arch == 'i386':
            vw.addAnalysisModule("vivisect.analysis.i386.importcalls")
            vw.addAnalysisModule("vivisect.analysis.i386.golang")
        elif arch == 'amd64':
            vw.addAnalysisModule("vivisect.analysis.amd64.golang")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")
        vw.addFuncAnalysisModule("vivisect.analysis.ms.hotpatch")
        vw.addFuncAnalysisModule("vivisect.analysis.ms.msvc")

        if arch in ('i386', 'amd64'):
            # Applies to i386 and amd64, will split it up when neccessary
            vw.addFuncAnalysisModule("vivisect.analysis.i386.instrhook")

        # Snap in an architecture specific emulation pass
        if arch == 'i386':
            vw.addFuncAnalysisModule("vivisect.analysis.i386.calling")

        elif arch == 'amd64':
            vw.addFuncAnalysisModule("vivisect.analysis.amd64.emulation")

        elif arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        # See if we got lucky and got arg/local hints from symbols
        vw.addAnalysisModule('vivisect.analysis.ms.localhints')
        # Find import thunks
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.noret")
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule('vivisect.analysis.ms.msvcfunc')
        vw.addAnalysisModule("vivisect.analysis.generic.thunks")

        vw.addAnalysisModule('vivisect.analysis.generic.strconst')

    elif fmt == 'elf':  # ELF ########################################################

        # elfplt wants to be run before generic.entrypoints.
        vw.addAnalysisModule("vivisect.analysis.elf.elfplt")
        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        vw.addAnalysisModule("vivisect.analysis.elf")

        if arch in ('i386', 'amd64', 'arm', 'aarch64', 'a64'):
            vw.addImpApi('posix', arch)

        if arch == 'i386':
            viv_analysis_i386.addEntrySigs(vw)
            vw.addAnalysisModule("vivisect.analysis.i386.importcalls")
            # add va set for tracking thunk_reg function(s)
            vw.addFuncAnalysisModule("vivisect.analysis.i386.thunk_reg")

        elif arch in ARM_ARCHS:
            vw.addFuncAnalysisModule('vivisect.analysis.arm.thunk_reg')
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        vw.addAnalysisModule("vivisect.analysis.generic.relocations")
        vw.addAnalysisModule("vivisect.analysis.elf.libc_start_main")
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule("vivisect.analysis.generic.pointertables")
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")

        # Generic code block analysis
        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")

        if arch in ('i386', 'amd64'):
            # Applies to i386 and amd64, will split it up when neccessary
            vw.addFuncAnalysisModule("vivisect.analysis.i386.instrhook")

        # Add our emulation modules
        if arch == 'i386':
            vw.addFuncAnalysisModule("vivisect.analysis.i386.calling")

        elif arch == 'amd64':
            vw.addFuncAnalysisModule("vivisect.analysis.amd64.emulation")

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        elif arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")


        # we want emulation to have completed so we know what the Args look like
        vw.addFuncAnalysisModule("vivisect.analysis.generic.symswitchcase")

        # Find import thunks
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.noret")
        # due to inconsistencies in plt layouts, we'll keep this as a func module as well
        vw.addFuncAnalysisModule("vivisect.analysis.elf.elfplt")
        # late-analysis ELF PLT tidying up, allowing unused PLT entries to be made into functions
        vw.addAnalysisModule("vivisect.analysis.elf.elfplt_late")
        vw.addAnalysisModule("vivisect.analysis.generic.linker")
        vw.addAnalysisModule("vivisect.analysis.generic.thunks")
        vw.addAnalysisModule("vivisect.analysis.generic.pointers")

    elif fmt == 'macho': # MACH-O ###################################################

        vw.addAnalysisModule("vivisect.analysis.generic.linker")
        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        if arch == 'i386':
            viv_analysis_i386.addEntrySigs(vw)
            vw.addAnalysisModule("vivisect.analysis.i386.importcalls")

        vw.addStructureModule('osx', 'vstruct.defs.macho')

        # Add the one that brute force finds function entry signatures
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule("vivisect.analysis.generic.relocations")
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")
        vw.addAnalysisModule("vivisect.analysis.generic.pointertables")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")

        if arch == 'i386':
            vw.addFuncAnalysisModule("vivisect.analysis.i386.calling")

        elif arch == 'amd64':
            vw.addFuncAnalysisModule("vivisect.analysis.amd64.emulation")

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        elif arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        vw.addFuncAnalysisModule("vivisect.analysis.generic.symswitchcase")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")
        vw.addAnalysisModule("vivisect.analysis.generic.pointers")

    elif fmt == 'blob': # BLOB ######################################################

        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule("vivisect.analysis.generic.pointertables")
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")

        if arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.symswitchcase")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")

    elif fmt in ('ihex', 'srec'): # Intel HEX  or SRECORD (similar) #################

        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule("vivisect.analysis.generic.pointertables")
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")

        if arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.symswitchcase")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")

    elif fmt == 'vbf': # VBF ######################################################
        if arch in PPC_ARCHS:
            # Initial VLE pages must be managed manually for bare
            # metal/embedded PowerPC targets
            vw.addAnalysisModule("vivisect.analysis.ppc.vlepages")

            # potentially tags a new EntryPoint, so must preceed entrypoints
            vw.addAnalysisModule("vivisect.analysis.ppc.bootstrap")

        vw.addAnalysisModule("vivisect.analysis.generic.entrypoints")
        vw.addAnalysisModule("vivisect.analysis.generic.funcentries")
        vw.addAnalysisModule("vivisect.analysis.generic.relocations")
        vw.addAnalysisModule("vivisect.analysis.generic.pointertables")
        vw.addAnalysisModule("vivisect.analysis.generic.emucode")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.codeblocks")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.impapi")

        if arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        if arch in ARM_ARCHS:
            vw.addFuncAnalysisModule("vivisect.analysis.arm.emulation")
            vw.addFuncAnalysisModule('vivisect.analysis.arm.renaming')

        elif arch in ('aarch64', 'a64'):
            vw.addFuncAnalysisModule("vivisect.analysis.aarch64.emulation")

        vw.addFuncAnalysisModule("vivisect.analysis.generic.symswitchcase")
        vw.addFuncAnalysisModule("vivisect.analysis.generic.thunks")

    else:

        raise Exception('Analysis modules unknown for format: %s' % fmt)

    logger.info('Vivisect Analysis Setup Hooks Complete')



'''
A unified context menu builder for all address context clicks.
'''
from PyQt5.QtWidgets import QMenu

import envi
import envi.bits as e_bits
import envi.qt.memcanvas as e_q_memcanvas
import vivisect.analysis.generic.codeblocks as vagc

from vqt.common import *
from envi.threads import *
from vivisect.const import *


@firethread
def printEmuState(vw, fva, va):
    vw.vprint('Running emulator to: 0x%.8x' % (va,))
    emu = vw.getEmulator()

    stack = emu.getStackCounter()
    emu.runFunction(fva, stopva=va, maxhit=2)
    dstack = emu.getStackCounter() - stack

    vw.vprint("Showing Register/Magic State At: 0x%.8x" % va)
    vw.vprint('Stack Delta: %d' % dstack)

    # FIXME: this may not be as flexible as it could be, as we don't necessarily *have* to have a location...
    # though we certainly should.
    loc = vw.getLocation(va)
    if loc is None:
        vw.vprint("ARG! can't find location info for 0x%x" % va)
        return

    lva, lsz, ltype, tinfo = loc
    op = vw.parseOpcode(va, arch=tinfo)
    vw.canvas.addVaText("0x%.8x: " % va, va)
    op.render(vw.canvas)
    vw.canvas.addText("\n")
    for i in range(len(op.opers)):
        o = op.opers[i]
        o.render(vw.canvas, op, i)
        oaddr = o.getOperAddr(op, emu)
        if oaddr is not None:
            vw.canvas.addText(' [ 0x%x ] ' % oaddr)

        vw.canvas.addText(" = ")
        oval = o.getOperValue(op, emu)
        taint = emu.getVivTaint(oval)
        base = "0x%.8x (%d)" % (oval, oval)
        if taint is not None:
            base += '%s + %d' % (emu.reprVivTaint(taint), oval - taint[0])

        vw.vprint(base)

@firethread
def newMemoryView(vw, xexpr):
    try:
        vw.getVivGui().newMemoryView(name="mem:%s" % xexpr, expr=xexpr)
        
    except:
        logger.warning("Failed to create new MemoryView", exc_info=1)

@firethread
def newFuncGraph(vw, xexpr):
    try:
        vw.getVivGui().newFuncGraphView(name="FG:%s" % xexpr, expr=xexpr)
        
    except:
        logger.warning("Failed to create new FunctionGraph", exc_info=1)


def initMemSendtoMenu(vw, xexpr, xmenu):
    e_q_memcanvas.initMemSendtoMenu(xexpr, xmenu)
    submenu = xmenu.addMenu('sendto: <new>')
    submenu.addAction('Memory View', ACT(newMemoryView, vw, xexpr))

    # check if it's code
    tgt = vw.parseExpression(xexpr)
    if tgt is None:
        return

    funcva = vw.getFunction(tgt)
    if funcva is None:
        return
    
    submenu.addAction('FuncGraph View', ACT(newFuncGraph, vw, xexpr))

def buildContextMenu(vw, va=None, expr=None, menu=None, parent=None, nav=None, tag=None):
    '''
    Return (optionally construct) a menu to use for handling a context click
    at the given virtual address or envi expression.

    Arguments:
    va      - virtual address of the context click
    expr    - expression to parse instead of va
    menu    - existing menu to add items to
    parent  - Qt parent
    nav     - the "local" EnviNavMixin instance
    '''
    if va is None:
        va = vw.parseExpression(expr)

    if expr is None:
        expr = '0x%.8x' % va

    if menu is None:
        menu = QMenu(parent=parent)

    menu.addAction('rename (n)', ACT(vw.getVivGui().setName, va, tag))
    menu.addAction('comment (;)', ACT(vw.getVivGui().setVaComment, va))
    menu.addAction('print location', ACT(vw.getVivGui().getLocation, va))

    refsto = vw.getXrefsTo(va)
    refsfrom = vw.getXrefsFrom(va)

    if refsto:
        rtomenu = menu.addMenu('xrefs to')
        for fromva, tova, xrtype, xrflag in refsto:
            xloc = vw.getLocation(fromva)
            xexpr = '0x%.8x' % fromva
            xrepr = '0x%.8x: %s' % (fromva, vw.reprLocation(xloc))
            xfva = vw.getFunction(fromva)
            if xfva is not None:
                xrepr = '%s (%s)' % (xrepr, vw.getName(xfva))
            xmenu = rtomenu.addMenu(xrepr)
            if nav:
                xmenu.addAction('(this window)', ACT(nav.enviNavGoto, xexpr))
            initMemSendtoMenu(vw, xexpr, xmenu)

    if refsfrom:
        rfrmenu = menu.addMenu('xrefs from')
        for fromva, tova, xrtype, xrflag in refsfrom:
            xloc = vw.getLocation(tova)
            xexpr = '0x%.8x' % tova
            xrepr = '0x%.8x: %s' % (tova, vw.reprLocation(xloc))
            xfva = vw.getFunction(tova)
            if xfva is not None:
                xrepr = '%s (%s)' % (xrepr, vw.getName(xfva))
            xmenu = rfrmenu.addMenu(xrepr)
            if nav:
                xmenu.addAction('(this window)', ACT(nav.enviNavGoto, xexpr))
            initMemSendtoMenu(vw, xexpr, xmenu)

    fva = vw.getFunction(va)
    if fva is not None:
        funcmenu = menu.addMenu('function')
        funcname = vw.getName(fva)
        if nav:
            funcmenu.addAction(funcname[:80], ACT(nav.enviNavGoto, funcname))

        rtype, rname, cconv, cname, cargs = vw.getFunctionApi(fva)
        if cargs:
            argmenu = funcmenu.addMenu('args')
            for i, (atype, aname) in enumerate(cargs):
                act = ACT(vw.getVivGui().setFuncArgName, fva, i, atype, aname)
                argmenu.addAction(aname, act)

        funclocals = vw.getFunctionLocals(fva)
        if funclocals:
            funclocals.sort()
            funclocals.reverse()

            locmenu = funcmenu.addMenu('locals')

            for _, spdelta, ltype, linfo in funclocals:
                if spdelta > 0: # FIXME perhaps make this flexable based on cconv?
                    continue

                # Make the workspace do the resolving for us
                typename, varname = vw.getFunctionLocal(fva,spdelta)
                act = ACT(vw.getVivGui().setFuncLocalName, fva, spdelta, typename, varname)
                locmenu.addAction(varname, act)

        if fva != va:
            funcmenu.addAction('show emulator state', ACT(printEmuState, vw, fva, va))

        funcmenu.addAction('call graph', ACT(vw.getVivGui().showFuncCallGraph, fva))
        funcmenu.addAction('re-analyze codeblocks', ACT(vagc.analyzeFunction, vw, fva))
        if fva == va:
            # funcmenu.addAction('delete function', ACT(vw.delFunction, va))
            funcmenu.addAction('delete function', ACT(vw.getVivGui().delFunction, va))

    loc = vw.getLocation(va)
    if loc is None:
        makemenu = menu.addMenu('make')
        makemenu.addAction('code (c)', ACT(vw.makeCode, va))
        makemenu.addAction('function (f)', ACT(vw.makeFunction, va))
        makemenu.addAction('string (s)', ACT(vw.makeString,  va))
        makemenu.addAction('pointer (p)', ACT(vw.makePointer, va))
        makemenu.addAction('unicode (u)', ACT(vw.makeUnicode, va))
        makemenu.addAction('structure (S)', ACT(vw.getVivGui().makeStruct, va))
        makemenu.addAction('pointer array (P)', ACT(vw.getVivGui().makePtrArray, va))

        nummenu = makemenu.addMenu('number')
        for size in (1, 2, 4, 8):
            nummenu.addAction("%d-bit (%d bytes)" % (size << 3, size), ACT(vw.makeNumber, va, size=size))

        archmenu = makemenu.addMenu('code (archs)')
        prevumenu = menu.addMenu('preview instruction')

        archs = [ (archname,archid) for (archid,archname) in envi.getArchNames().items() ]
        archs.sort()
        for archname,archid in archs:
            if archname == 'default':
                continue
            archmenu.addAction(archname, ACT(vw.makeCode, va, arch=archid))
            prevumenu.addAction(archname, ACT(vw.previewCode, va, arch=archid))

    else:

        if loc[L_LTYPE] == LOC_OP:

            op = vw.parseOpcode(va, arch=loc[L_TINFO])
            for idx,oper in enumerate(op.opers):
                # Give the option to switch ('hint') that you want
                # the immediate operand displayed differently...
                if oper.isImmed():
                    val = oper.getOperValue(op)
                    hval = e_bits.hex(val)

                    cval = val
                    r = []
                    while cval:
                        r.append(chr(cval & 0xff))
                        cval = cval >> 8
                    cstr = repr(''.join(r))

                    immmenu = menu.addMenu('immediate')
                    # FIXME struct offsets?
                    immmenu.addAction('decimal (%d)' % val, ACT(vw.setSymHint, va, idx, str(val)))
                    immmenu.addAction('hex (%s)' % hval,    ACT(vw.setSymHint, va, idx, hval))
                    immmenu.addAction('chars (%s)' % cstr,  ACT(vw.setSymHint, va, idx, cstr))

                    names = vw.vsconsts.revLookup(val)
                    if names is not None:
                        for name in names:
                            immmenu.addAction(name, ACT(vw.setSymHint, va, idx, name))
            menu.addAction('make code xref->', ACT(vw.getVivGui().addVaXref, va))

            if vw.getVaSetRow('SwitchCases_TimedOut', va):
                do_analyze = ACT(vw.getVivGui().reanalyzeSwitchCase, va)
                do_analyze.setNewThread()
                menu.addAction('Reanalyze Switchcase (timed out)', do_analyze)


        menu.addAction('bookmark (B)',   ACT(vw.getVivGui().addBookmark, va))
        menu.addAction('undefine (U)',   ACT(vw.delLocation, va))

    initMemSendtoMenu(vw, expr, menu)

    # give any extensions a chance to play
    for extname, exthook in vw._ext_ctxmenu_hooks.items():
        logger.info('exthook: %r', exthook)
        exthook(vw, va, expr, menu, parent, nav, tag)

    return menu



import copy

import envi.bits as e_bits

from vivisect.const import *
from vivisect.symboliks.common import *
from vivisect.symboliks.expression import symexp

def ismatch(sym,tmp):
    '''
    FIXME - eventually optimize this into one match call
            against a set of trees....

    Given a sym and s "symbolik template" determine if
    the given sym matches the specified template.

    A symbolik "template" declares a symbolik expression
    which represents a given combination of consts, non-consts,
    operators, and spcific values using the following rules:

        ( where N is a number chosen to unique the variable )

        * cN - any constant expression
        * xN - any non-constant expression
        * <num> - A specific constant value
        * <oper> - An exactly matching operator

    For example

        truths:
        ismatch( symexp('eax + 20'), symexp('x1 + c1'))
        ismatch( symexp('(eax + 20) + ebx'), symexp('(x1 + c1) + x2'))

        falses:
        ismatch( symexp('eax + 20'), symexp('x1 + 30'))
        ismatch( symexp('eax + 20'), symexp('x1 + x2'))
        ismatch( symexp('eax - 20'), symexp('x1 + c1'))

    Returns: None or a dict of template var names to sym
             objects ( Const instances are replaced with
             python integer values to symplify reduction
             syntax...

        ismatch( symexp('eax + 20'), symexp('x1 + c1')) -> {'x1':Var('eax'), 'c1':20}
    '''
    todo = [(sym, tmp)]

    symwidth = sym.getWidth()

    ret = {}
    while todo:
        s, t = todo.pop()

        if s.getWidth() != symwidth and s.symtype != SYMT_CONST:
            return None

        if t.symtype == SYMT_VAR:
            if t.name.startswith('c') and s.symtype != SYMT_CONST:
                return None

            if t.name.startswith('x') and s.symtype == SYMT_CONST:
                return None

            # haz? if so, require equality...
            haz = ret.get(t.name)
            if haz and haz.solve() != s.solve():
                return None

            ret[t.name] = s
            continue

        if t.symtype != s.symtype:
            return None

        if t.symtype & SYMT_OPER:
            todo.append((s.kids[0], t.kids[0]))
            todo.append((s.kids[1], t.kids[1]))
            continue

        if t.symtype in (SYMT_CON_EQ, SYMT_CON_NE):
            todo.append((s.kids[0], t.kids[0]))
            todo.append((s.kids[1], t.kids[1]))
            continue

        if t.symtype == SYMT_CONST:
            if t.value != s.value:
                return None
            continue

        raise Exception(str(t))

    # replace all consts with their solved values...
    for k, v in ret.items():
        if k.startswith('c'):
            ret[k] = v.solve()

    return ret

def variants(sym):
    '''
    Given a SymbolikBase, generate all of the comutative
    combinations of the given expression.

    Example:

        variants( symexpr('(x + 20) + y') )

        Would return a list of sym objects with the
        given structure:

            (x + 20) + y
            (x + y) + 20
            (y + 20) + x
            (y + x) + 20
    '''
    todo = [sym]
    swaps = []

    while todo:
        t = todo.pop()
        if t.commutative:
            k0 = list(t.kids)
            k1 = list(t.kids)
            k1.reverse()
            swaps.append((t, (k0, k1)))

        todo.extend(t.kids)

    for i in range(2**len(swaps)):
        for t, k in swaps:
            t.kids[0:2] = k[i & 1]
            i >>= 1

        sym.clearCache()
        yield copy.deepcopy(sym)  # s.clone()

def addsub(x, c):
    '''
    Given the sign of c, return either an o_add or
    an o_sub with x on the left and abs(c) on the right.

    ( or just x if c happens to be 0 )
    '''
    if c > 0:
        return x + Const(c, x.getWidth())
    elif c < 0:
        return x - Const(abs(c), x.getWidth())
    else:
        return x

def sub_c_x(c, x):
    '''
    Construct an o_sub for c - x.
    If c is negative, build (0 - c) - x
    '''
    xwidth = x.getWidth()
    if c < 0:
        return (Const(0, xwidth) - Const(c, xwidth)) - x
    return Const(c,xwidth) - x

def ormask(v,c):
    vwidth = v.getWidth()
    if c == 0:
        return v

    if e_bits.u_maxes[vwidth] == c:
        return e_bits.u_maxes[vwidth]

    return v | Const(c,vwidth)

def andmask(v,c):
    vwidth = v.getWidth()
    if c == 0:
        return Const(0,vwidth)

    umax = e_bits.u_maxes[vwidth]
    if umax == c:
        return v

    if umax & c == 0:
        return Const(0,vwidth)

    return v & Const(c,vwidth)

def mulbase(v, c):
    vwidth = v.getWidth()
    if c == 0:
        return Const(0, vwidth)
    if c == 1:
        return v
    return v * Const(c,vwidth)

def divbase_vc(v, c):
    vwidth = v.getWidth()
    if c == 1:
        return v
    return v / Const(c,vwidth)

def divbase_cv(c, v):
    vwidth = v.getWidth()
    if c == 0:
        return Const(0,vwidth)
    return Const(c,vwidth) / v

def muldiv(v, m, d):
    vwidth = v.getWidth()

    if m == d:
        return v

    if m % d == 0:
        return v * Const(int(m/d), vwidth)

def xpandrules(rules):
    reducers = []
    for symtmp, reducer in rules:
        for symtmp in variants(symexp(symtmp)):
            reducers.append((symtmp, reducer))
    return reducers

def symneg(x):
    return Const(0, x.getWidth()) - x

# NOTE: all reducers *must* be declared in most->least
# specific order.
reducers = {
    SYMT_OPER_ADD: xpandrules([
        ('(x1 - c1) + (x2 - c2)', lambda m, emu=None: addsub(m['x1'] + m['x2'], -(m['c1'] + m['c2']))),
        ('(c1 - x1) + (x2 + c2)', lambda m, emu=None: addsub(m['x2'] - m['x1'], m['c1'] + m['c2'])),
        ('(x1 + c1) + (x2 - c2)', lambda m, emu=None: addsub(m['x1'] + m['x2'], m['c1'] - m['c2'])),
        ('(x1 + c1) + (x2 + c2)', lambda m, emu=None: addsub(m['x1'] + m['x2'], m['c1'] + m['c2'])),
        ('(x1 + c1) + c2', lambda m, emu=None: addsub(m['x1'], m['c1'] + m['c2'])),
        ('(x1 + c1) + x2', lambda m, emu=None: addsub(m['x1'] + m['x2'], m['c1'])),
        ('(x1 - c1) + c2', lambda m, emu=None: addsub(m['x1'], m['c2'] - m['c1'])),
        ('(c1 - x1) + x2', lambda m, emu=None: addsub(m['x2'] - m['x1'], m['c1'])),
        ('(c1 - x1) + c2', lambda m, emu=None: addsub(symneg(m['x1']), m['c1'] + m['c2'])),
        ('(x1 - x2) + x2', lambda m, emu=None: m['x1']),
        ('(x1 + 0)', lambda m, emu=None: m['x1']),
        ('(x1 + x1)', lambda m, emu=None: m['x1'] * Const(2, m['x1'].getWidth())),
        ('(x1 + x2) + x1', lambda m, emu=None: m['x2'] + (m['x1'] * Const(2,m['x1'].getWidth()))),
        ('(x1 * c1) + x1', lambda m, emu=None: m['x1'] * Const( m['c1'] + 1, m['x1'].getWidth())),
    ]),

    SYMT_OPER_SUB: xpandrules([
        ('(x1 - c1) - (x2 - c2)',
            lambda m,emu=None: addsub(m['x1'] - m['x2'], -(m['c1']-m['c2']))), # ((x1 - c1) + c2) - x2 # x1 - (c1 - c2) - x2 # (x1 - x2) - (c1 - c2)
        ('(x1 - c1) - (x2 + c2)', lambda m,emu=None: addsub(m['x1'] - m['x2'], -(m['c1'] + m['c2']))), # x1 - (c1 + c2) - x2 # (x1 - x2) - (c1 + c2)
        ('(c1 - x1) - (x2 + c2)', # (0 - x1) + c1 - x2 - c2 # ((0 - x1) - x2) + (c1-c2)
            lambda m,emu=None: addsub(symneg(m['x1']) - m['x2'], m['c1'] - m['c2'])),
        ('(x1 + c1) - (x2 - c2)', lambda m,emu=None: addsub(m['x1'] - m['x2'], m['c1'] + m['c2'])), # x1 + (c1 + c2) - x2 # (x1 - x2) + (c1 + c2)
        ('(x1 + c1) - (x2 + c2)', lambda m,emu=None: addsub(m['x1'] - m['x2'], m['c1'] - m['c2'])), # x1 + (c1 - c2) - x2 # (x1 - x2) + (c1 - c2)
        #('(x + x) - (x * 2)', lambda m,emu=None:

        ('(x1 + c1) - c2', lambda m,emu=None: addsub(m['x1'], m['c1'] - m['c2'])),  # x1 + (c1 - c2) #
        ('(x1 - c1) - c2', lambda m,emu=None: addsub(m['x1'], -(m['c1'] + m['c2']))),  # x1 - (c1 + c2) #
        ('(c1 - x1) - c2', lambda m,emu=None: sub_c_x(m['c1'] - m['c2'], m['x1'])), # (c1 - c2) - x1 #
        ('(x1 + x2) - x2', lambda m, emu=None: m['x1']),

        ('c1 - (x1 - c2)', lambda m,emu=None: sub_c_x(m['c1'] + m['c2'], m['x1'])), # (c1 + c2) - x1 # 
        ('c1 - (c2 - x1)', lambda m,emu=None: addsub(m['x1'], m['c1'] - m['c2'])),#  (c1 + x1) - c2 # (x1 + c1) - c2 # x1 + (c1 - c2) #
        ('c1 - (x1 + c2)', lambda m,emu=None: sub_c_x(m['c1'] - m['c2'], m['x1'])), #  (c1 - c2) - x1 # 

        ('(x1 - 0)', lambda m,emu=None: m['x1']),
        ('(x1 - x1)', lambda m,emu=None: 0),
    ]),

    SYMT_OPER_AND: xpandrules([
        ('(x1 & c1) & (x2 & c2)', lambda m,emu=None: andmask(m['x1'] & m['x2'], m['c1'] & m['c2'])),
        ('(x1 & c1) & c2', lambda m,emu=None: andmask(m['x1'], m['c1'] & m['c2'])),
        ('(x1 & c1)', lambda m,emu=None: andmask(m['x1'], m['c1'])),
        ('(x1 & x1)', lambda m,emu=None: m['x1']),
    ]),

    SYMT_OPER_OR: xpandrules([
        ('(x1 | c1) | (x2 | c2)', lambda m,emu=None: ormask(m['x1'] | m['x2'], m['c1'] | m['c2'])),
        ('(x1 | c1) | c2', lambda m,emu=None: ormask(m['x1'], m['c1'] | m['c2'])),
        ('(x1 | c1)', lambda m,emu=None: ormask(m['x1'], m['c1'])),
        ('(x1 | x1)', lambda m,emu=None: m['x1']),
    ]),

    SYMT_OPER_XOR: xpandrules([
        ('(x1 ^ c1) ^ c2', lambda m,emu=None: m['x1'] ^ Const( m['c1'] ^ m['c2'], m['x1'].getWidth())),
        ('(x1 ^ x1)', lambda m,emu=None: 0),
        ('(x1 ^ 0)', lambda m,emu=None: m['x1']),
    ]),

    SYMT_OPER_MUL: xpandrules([
        ('(x1 * c1) * (x2 * c2)', lambda m,emu=None: mulbase(m['x1'] * m['x2'], m['c1'] * m['c2'])),
        # NOT OK ('(x1 / c1) * c2', lambda m,emu=None: muldiv(m['x1'],m['c2'],m['c1'])),
        ('(x1 * c1) * c2', lambda m,emu=None: mulbase(m['x1'], m['c1'] * m['c2'])),
        ('(x1 * c1)', lambda m,emu=None: mulbase(m['x1'], m['c1'])),
    ]),

    SYMT_OPER_DIV: xpandrules([
        ('(x1 * c1) / c2', lambda m, emu=None: muldiv(m['x1'],m['c1'],m['c2'])),
        ('(x1 / c1)', lambda m, emu=None: divbase_vc(m['x1'], m['c1'])),
        ('(c1 / x1)', lambda m, emu=None: divbase_cv(m['c1'], m['x1'])),
        ('(x1 / x1)', lambda m, emu=None: 1),
    ]),

    SYMT_OPER_POW: xpandrules([
        ('(x1 ** 1)', lambda m, emu=None: m['x1']),
        ('(1 ** x1)', lambda m, emu=None: 1),
        ('(x1 ** 0)', lambda m, emu=None: 1),
        ('(0 ** x1)', lambda m, emu=None: 0),
    ]),

    SYMT_OPER_RSHIFT: xpandrules([
        ('(x1 >> 0)', lambda m, emu=None: m['x1']),
        ('(0 >> x1)', lambda m, emu=None: 0),
        ('(x1 >> c1)', lambda m, emu=None: divbase_vc(m['x1'], 2**m['c1'])),
    ]),

    SYMT_OPER_LSHIFT: xpandrules([
        ('(x1 << 0)', lambda m, emu=None: m['x1']),
        ('(0 << x1)', lambda m, emu=None: 0),
        ('(x1 << c1)', lambda m, emu=None: mulbase(m['x1'], 2**m['c1'])),
    ]),

    SYMT_CON_EQ: xpandrules([
        ('(x1 == x1)', lambda m, emu=None: 1),
    ]),

    SYMT_CON_NE: xpandrules([
        ('(x1 != x1)', lambda m, emu=None: 0),
    ]),
}

def reduceoper(sym,emu=None):
    '''
    Apply the current set of operator reducers to the given
    SymbolikBase.  Sym *must* be an instance of the
    Operator(SymbolikBase) class..
    '''
    if not reducers.get(sym.symtype):
        return
    for symtmp, reducer in reducers.get(sym.symtype):
        m = ismatch(sym, symtmp)
        if m is not None:
            ret = reducer(m, emu=emu)
            # do this to much simplify reducers...
            if type(ret) is int:
                ret = Const(ret,sym.getWidth())
            return ret



def analyze(vw):
    for fva in vw.getFunctions():
        analyzeFunction(vw, fva)


def analyzeFunction(vw, fva):
    fakename = vw.getName(fva+1)
    if fakename is not None:
        vw.makeName(fva+1, None)
        vw.makeName(fva, fakename)




"""
Vamp is a function/codeblock signaturing framework which is
a subcomponent of vivisect.  These may be used to import/export
signature sets and potentially identify code reuse or static
linking...

Current signature ideas:
    function arg count
    code block count
    globals refs
    code block refs
    unusual instruction use
    odd immediates
    import calls
    other signature calls
    certianty index
    Exception handling

    There will be function characteristics and code-block
    characteristics...

NOTE: Initial signature code consists entirely of the envi
bytesig module and byte/mask sets for known function signatures.
"""


class Signature:
    """
    A function/procedure signature.
    """
    pass

from vivisect.const import *

def genSigAndMask(vw, funcva):

    """
    Generate an envi bytesig signature and mask for the given
    function block.  This will properly mask off relocations
    if present.
    """

    fsize = 0
    if funcva not in vw.getFunctions():
        funcva = vw.getFunction(funcva)
        if funcva is None:
            raise Exception('Given funcva not a function or within a known function')
    func_blocks = [cbva for cbva, _, _ in vw.getFunctionBlocks(funcva)]
    # Figure out the size of the first linear chunk
    # in this function...
    cb = vw.getCodeBlock(funcva)
    if cb[CB_VA] not in func_blocks:
        raise Exception("funcva not in given func")
    while cb is not None:
        cbva, cbsize, cbfunc = cb
        if cbfunc != funcva:
            break
        fsize += cbsize
        cb = vw.getCodeBlock(cbva+cbsize)

    if fsize == 0:
        raise Exception("0 length function??!?1")

    bytez = vw.readMemory(funcva, fsize)

    sig = b""
    mask = b""
    i = 0
    while i < fsize:
        rtype = vw.getRelocation(funcva + i)
        if rtype is None:
            sig += bytez[i:i+1]
            mask += b"\xff"
            i += 1
        elif rtype == RTYPE_BASERELOC:
            x = b"\x00" * vw.psize
            sig += x
            mask += x
            i += vw.psize
        else:
            raise Exception("Unhandled Reloc Type: %d" % rtype)

    return sig, mask



import os
import binascii
import unittest

import vstruct
import vstruct.cparse as vs_cparse
import vstruct.builder as vs_builder


csrc_simple = '''
struct monty {
    int life;
    char of;
    long brian;
};
'''

csrc_nested = '''
struct python {
    unsigned short meaning;
    struct andthe {
        unsigned long int of;
        char life[20];
    } holygrail;
    int hollywood[64];
    unsigned char bow[5];
};
'''


class VstructBuilderTest(unittest.TestCase):

    def test_init(self):
        simple = vs_cparse.ctorFromCSource(csrc_simple)
        nested = vs_cparse.ctorFromCSource(csrc_nested)

        bldr = vs_builder.VStructBuilder()
        bldr.addVStructCtor('pe.simple', simple)
        bldr.addVStructCtor('elf.nested', nested)
        self.assertEqual(set(['pe.simple', 'elf.nested']), set(bldr.getVStructNames()))

        bldr.delVStructCtor('elf.nested')
        self.assertEqual(['pe.simple'], bldr.getVStructNames())

    def test_constructor(self):
        pass

    def test_namespaces(self):
        bldr = vs_builder.VStructBuilder()
        subb = vs_builder.VStructBuilder()

        bldr.addVStructNamespace('subname', subb)
        bldr.addVStructNamespace('dupname', subb)
        self.assertTrue(bldr.hasVStructNamespace('subname'))
        self.assertEqual(set(bldr.getVStructNamespaces()), set([('subname', subb), ('dupname', subb)]))
        self.assertEqual(set(bldr.getVStructNamespaceNames()), set(['subname', 'dupname']))

        nested = vs_cparse.ctorFromCSource(csrc_nested)
        bldr.addVStructCtor('foo', nested)
        self.assertEqual(bldr.getVStructCtorNames(), ['foo'])

    def test_get_pycode(self):
        # TODO: Windows CI
        pass



"""
Breakpoint Objects
"""

# Copyright (C) 2007 Invisigoth - See LICENSE file for details

import time
import logging
from collections import defaultdict

import vtrace


logger = logging.getLogger(__name__)

class Breakpoint:
    """
    Breakpoints in Vtrace are platform independant objects that
    use the underlying trace objects to get things like the
    program counter and the break instruction.  As long as
    platforms are completely implemented, all breakpoint
    objects should be portable.
    """

    bpcodeobj = {} # Cache compiled code objects on the class def

    def __init__(self, address, expression=None):
        self.resonce = False        # has this addr expression been resolved yet?
        self.address = address
        self.enabled = True         # should this BP be used/ignored
        self.active = False         # have we placed a BP in the code (eg. i386: \xCC)
        self.silent = False         # don't print "Hit Break" messages, still runs code/notifiers
        self.fastbreak = False      # no NOTIFY_BREAK, autocont, no NOTIFY_CONTINUE
        self.stealthbreak = False   # no NOTIFY_BREAK - used for hidden/system events
        self._complained = False    # only complain about not resolving this *once*

        self.id = -1
        self.vte = None
        self.bpcode = None
        if expression:
            self.vte = expression

    def getAddress(self):
        """
        This will return the address for this breakpoint.  If the return'd
        address is None, this is a deferred breakpoint which needs to have
        resolveAddress() called to attempt to set the address.
        """
        return self.address

    def getId(self):
        return self.id

    def getName(self):
        if self.vte:
            return str(self.vte)
        return "0x%.8x" % self.address

    def __repr__(self):
        if self.address is None:
            addr = "unresolved"
        else:
            addr = "0x%.8x" % self.address
        return "[%d] %s %s: %s" % (self.id, addr, self.__class__.__name__, self.getName())

    def inittrace(self, trace):
        '''
        A callback to do housekeeping at the time the breakpoint is
        added to the tracer object.  This should be used instead of activate
        for initialization time infoz to save on time per activate call...
        '''
        pass

    def activate(self, trace):

        if not self.active:
            trace.archActivBreakpoint(self.address)
            self.active = True

    def deactivate(self, trace):

        if self.active:
            trace.archClearBreakpoint(self.address)
            self.active = False

    def resolvedaddr(self, trace, addr):
        '''
        An initialization callback which will be executed when the
        actual address for this breakpoint has been resolved.
        '''

    def resolveAddress(self, trace):
        """
        Try to resolve the address for this break.  If this is a statically
        addressed break, just return the address.  If it has an "expression"
        use that to resolve the address...
        """
        if self.address is None and self.vte:
            try:
                self.address = trace.parseExpression(self.vte)

            except Exception as e:
                # this will happen with unresolved breakpoints.  
                # depending on when resolution happens, the library may not have loaded yet.
                if not self._complained:
                    logger.warning('Failed to resolve breakpoint address for expression: %s (delayed resolution?)', self.vte)
                    self._complained = True
                self.address = None

        # If we resolved, lets get our saved code...
        if self.address is not None and not self.resonce:
            self.resonce = True
            self.resolvedaddr(trace, self.address)

        return self.address

    def isEnabled(self):
        """
        Is this breakpoint "enabled"?
        """
        return self.enabled

    def setEnabled(self, enabled=True):
        """
        Set this breakpoints "enabled" status
        """
        self.enabled = enabled

    def setBreakpointCode(self, pystr):
        """
        Use this method to set custom python code to run when this
        breakpoint gets hit.  The code will have the following objects
        mapped into it's namespace when run:
            trace - the tracer
            vtrace - the vtrace module
            bp - the breakpoint
        """
        self.bpcode = pystr
        Breakpoint.bpcodeobj.pop(self.id, None)

    def getBreakpointCode(self):
        """
        Return the current python string that will be run when this break is hit.
        """
        return self.bpcode

    def notify(self, event, trace):
        """
        Breakpoints may also extend and implement "notify" which will be
        called whenever they are hit.  If you want to continue the ability
        for this breakpoint to have bpcode, you must call this method from
        your override.
        """
        if self.bpcode is not None:
            cobj = Breakpoint.bpcodeobj.get(self.id, None)
            if cobj is None:
                fname = "BP:%d (0x%.8x)" % (self.id, self.address)
                cobj = compile(self.bpcode, fname, "exec")
                Breakpoint.bpcodeobj[self.id] = cobj

            d = vtrace.VtraceExpressionLocals(trace)
            d['bp'] = self
            d['event'] = event
            d['trace'] = trace
            d['vprint'] = trace.vprint
            if hasattr(trace, 'db'):
                d['db'] = trace.db
            exec(cobj, None, d)

class TrackerBreak(Breakpoint):
    """
    A breakpoint which will record how many times it was hit
    (by the address it was at) as metadata for the tracer.
    """
    def notify(self, event, trace):
        tb = trace.getMeta("TrackerBreak", None)
        if tb is None:
            tb = {}
        trace.setMeta("TrackerBreak", tb)
        tb[self.address] = (tb.get(self.address, 0) + 1)
        Breakpoint.notify(self, event, trace)

class OneTimeBreak(Breakpoint):
    """
    This type of breakpoint is exclusivly for marking
    and code-coverage stuff.  It removes itself.
    (most frequently used with a continued trace)
    """
    def notify(self, event, trace):
        trace.removeBreakpoint(self.id)
        Breakpoint.notify(self, event, trace)

class StopRunForeverBreak(Breakpoint):
    """
    This breakpoint will turn off RunForever mode
    on the tracer object when hit.  it's a good way
    to let things run on and on processing exceptions
    but stop when you get to this one thing.
    """
    def notify(self, event, trace):
        trace.setMode("RunForever", False)
        Breakpoint.notify(self, event, trace)

class StopAndRemoveBreak(Breakpoint):
    """
    When hit, take the tracer out of run-forever mode and
    remove this breakpoint.
    """
    def notify(self, event, trace):
        trace.setMode("RunForever", False)
        trace.removeBreakpoint(self.id)
        Breakpoint.notify(self, event, trace)

class CallBreak(Breakpoint):
    """
    A special breakpoint which will restore process
    state (registers in particular) when it gets hit.
    This is primarily used by the call method inside
    the trace object to restore original state
    after a successful "call" method call.

    Additionally, the endregs dict will be filled in
    with the regs at the time it was hit and kept until
    we get garbage collected...
    """
    def __init__(self, address, saved_regs):
        Breakpoint.__init__(self, address)
        self.endregs = None # Filled in when we get hit
        self.saved_regs = saved_regs

    def notify(self, event, trace):
        self.endregs = trace.getRegisters()
        trace.removeBreakpoint(self.id)
        trace.setRegisters(self.saved_regs)
        trace.setMeta("PendingSignal", None)

class SnapshotBreak(Breakpoint):
    """
    A special breakpoint type which will produce vtrace snapshots
    for the target process when hit.  The snapshots will be saved
    to a default name of <exename>-<timestamp>.vsnap.  This is not
    recommended for use in heavily hit breakpoints as taking a
    snapshot is processor intensive.
    """
    def notify(self, event, trace):
        exe = trace.getExe()
        snap = trace.takeSnapshot()
        snap.saveToFile("%s-%d.vsnap" % (exe, time.time()))
        Breakpoint.notify(self, event, trace)

class NiceBreakpoint(Breakpoint):
    '''
    Calls the underlying breakpoint constructor with the correct constructor
    automagically by checking the type you passed in (int vs other).
    '''
    def __init__(self, expr, *args, **kwargs):
        if isinstance(expr, int):
            vtrace.Breakpoint.__init__(self, expr, *args, **kwargs)
        else:
            vtrace.Breakpoint.__init__(self, None, expression=expr, *args, **kwargs)

def addHook(trace, expr, pre_callback, post_callback=None, cc=None, argc=None):
    '''
    Adds the specified pre and post callbacks to the specified expression.
    '''
    hbp = HookBreakpoint(expr, callingconv=cc, argc=argc)
    addr = hbp.resolveAddress(trace)

    # does a hook bp already exist in the deferred or active bplist?
    ret_bp = None
    if addr is None:
        for dbp in trace.deferred:
            if dbp.getName() == expr:
                ret_bp = dbp
                break
    else:
        ret_bp = trace.getBreakpointByAddr(addr)

    if ret_bp is None:
        # add a new bp, one does not exist at this location already
        trace.addBreakpoint(hbp)
        ret_bp = hbp
    elif not isinstance(ret_bp, HookBreakpoint):
        raise Exception('cannot add this hook, non-HookBreakpoint bp at this location')

    ret_bp.addPreHook(pre_callback)

    if post_callback is not None:
        ret_bp.addPostHook(post_callback)

class HookBreakpoint(NiceBreakpoint):
    '''
    A special breakpoint that allows pre/post handlers to be registered on a
    bp for a function.  Pre-handlers are executed at the time of the break.
    Post-handlers are implemented as a seperate breakpoint placed at the
    return address. (as read at the time of entry to the function, this does
    not handle things that manually mess with the return address within the
    function)

    Handlers are executed one at a time in an ordered sequence. Execution
    continues without breaking to the user.

    The prototype for pre hook callback handlers is:
    def prehook(event, trace, ret_addr, args, callconv)
    event - the event
    trace - trace object
    ret_addr - the return address (if calling convention is known)
    args - the function arguments (if calling convention is known)
    callconv - the calling convention object

    The prototype for post hook callback handlers is the same.
    def posthook(event, trace, saved_ret_addr, saved_args, callconv)
    '''
    def __init__(self, expr, callingconv=None, argc=None):
        vtrace.NiceBreakpoint.__init__(self, expr)

        self.fastbreak = True

        self.prehooks = []
        self.posthooks = []

        self.cc = callingconv
        self.argc = argc

        # holds call information by thread id
        # { tid : (ret addr, (arg0, arg1, ...) ), ... }
        self.callinfo = defaultdict(list)

        self.error_cb = self.defaultErrorHandler

    def defaultErrorHandler(self, hook_cb_name, stre):
        logger.error('Pre hook callback "%s" exception: %s', hook_cb_name, stre)

    def resolvedaddr(self, trace, addr):
        '''
        When we get resolved, lookup in impapi the calling convention and other
        details about the function.  Do not do this if we were explicitly told
        what to do.
        '''
        # told explicitly what to do, don't go look anything up
        if self.cc is not None and self.argc is not None:
            return

        # TODO: move this out of here after we move impapi to a top-level
        # package.
        import vivisect.impapi as viv_impapi
        # this code also exists in win32stealth, we should put this somewhere
        # common
        platform = trace.getMeta('Platform')
        arch = trace.getMeta('Architecture')
        self.impapi = viv_impapi.getImportApi(platform, arch)
        cc = self.impapi.getImpApiCallConv(self.vte)
        emu = vtrace.getEmu(trace)
        self.cc = emu.getCallingConvention(cc)
        apiargs = self.impapi.getImpApiArgs(self.vte)
        if apiargs is not None:
            self.argc = len(apiargs)

    def addPreHook(self, callback):
        self.prehooks.append(callback)

    def addPostHook(self, callback):
        self.posthooks.append(callback)

    def runPreHookCallbacks(self, hook_cbs, event, trace, ret_addr, args):
        for hook_cb in hook_cbs:
            try:
                hook_cb(event, trace, ret_addr, args, self.cc)
            except Exception as e:
                self.defaultErrorHandler(hook_cb, str(e))

    def notify(self, event, trace):
        ret_addr = None
        args = None
        if self.cc is not None:
            ret_addr = self.cc.getReturnAddress(trace)
            args = self.cc.getCallArgs(trace, self.argc)

            self.callinfo[trace.getCurrentThread()] = (ret_addr, args)

        # setup a PostHookBreakpoint on where we are headed to (if one is not
        # already there) we can't do this if we don't know the calling conv
        # information.
        if ret_addr is not None:
            ret_bp = trace.getBreakpointByAddr(ret_addr)
            if ret_bp is None:
                ret_bp = PostHookBreakpoint(ret_addr, self)
                trace.addBreakpoint(ret_bp)

            if not isinstance(ret_bp, PostHookBreakpoint):
                raise Exception('cannot add PostHookBreakpoint, another type of bp exists at this location')

        self.runPreHookCallbacks(self.prehooks, event, trace, ret_addr, args)


class PostHookBreakpoint(NiceBreakpoint):

    def __init__(self, expr, parent_hook_bp):
        vtrace.NiceBreakpoint.__init__(self, expr)
        self.parent = parent_hook_bp
        self.fastbreak = True

    def runPostHookCallbacks(self, event, trace, saved_ret_addr, saved_args):
        for hook_cb in self.parent.posthooks:
            try:
                hook_cb(event, trace, saved_ret_addr, saved_args, self.parent.cc)
            except Exception as e:
                logger.error('Post hook callback "%s" exception: %s', hook_cb, e)

    def notify(self, event, trace):
        tup = self.parent.callinfo.get(trace.getCurrentThread(), None)
        if tup is None:
            return

        ret_addr, args = tup

        self.runPostHookCallbacks(event, trace, ret_addr, args)

class PosixLibLoadHookBreakpoint(Breakpoint):
    '''
    POSIX systems need to hook DL to identfy when libraries are loaded.
    '''
    def __init__(self, expression):
        Breakpoint.__init__(self, None, expression=expression)
        self.stealthbreak = True

    def notify(self, event, trace):
        logger.debug("PosixLibLoadHookBreakpoint: reanalyze maps and resolve symbols")
        if not trace._findLibraryMaps(b'\x7fELF', always=True):
            # if we find new maps, we'll let the LOAD_LIBRARY Autoload config setting handle
            # whether we continue or not.  if we fire this and *don't* find a new map, 
            # let's just continue like nothing ever happened.  "Nothing to see here."
            trace.runAgain()



def splitline(pt1, pt2, percent=0.5):
    '''
    Return a point which splits the given line at the
    given percentage...

    Example: splitline( (0,0), (20, 30), 0.1)
    '''

    pt1_x, pt1_y = pt1
    pt2_x, pt2_y = pt2

    deltax = (pt2_x - pt1_x) * percent
    deltay = (pt2_y - pt1_y) * percent

    return int(pt1_x + deltax), int(pt1_y + deltay)

def calculate_bezier(points, steps = 30):
    '''
    Arbitrary depth and arbitrary precision bezier implementation.  Takes
    a list of (x,y) point tuples and returnes the points to draw for the
    bezier curve.
    '''
    ret = []
    points = [ (float(x),float(y)) for x,y in points ]

    for i in range(steps+1):

        pcent = i / float(steps)

        layers = [ points, ]
        while len(layers[-1]) != 1:
            l_points = layers[-1]
            newpoints = [ splitline( l_points[i], l_points[i+1], pcent) for i in range(len(l_points)-1) ]
            layers.append(newpoints)

        ret.append(layers[-1][0])

    return ret



import itertools
import traceback

import vqt.tree as vq_tree
import vqt.saveable as vq_save
import envi.qt.memory as e_q_memory
import envi.qt.memcanvas as e_q_memcanvas
import vivisect.tools.graphutil as viv_graph
import vivisect.symboliks.common as viv_sym_common
import vivisect.symboliks.effects as viv_sym_effects
import vivisect.symboliks.analysis as viv_sym_analysis
import vivisect.symboliks.expression as viv_sym_expression

from PyQt5 import QtCore
from PyQt5.QtWidgets import *

from vqt.main import *
from vqt.basics import *
from vivisect.const import *


class VivSymbolikPathsModel(vq_tree.VQTreeModel):
    columns = ('Path', 'Effect Count')


class VivSymbolikPathsView(vq_tree.VQTreeView):

    pathSelected = QtCore.pyqtSignal(object, object)

    def __init__(self, vw, parent=None):
        vq_tree.VQTreeView.__init__(self, parent=parent)
        self.setModel(VivSymbolikPathsModel(parent=self))

    def loadSymbolikPaths(self, paths):
        cnt = 0
        model = VivSymbolikPathsModel(parent=self)
        for i, (emu, effects) in enumerate(paths):
            model.append((str(i), len(effects), emu, effects))
            cnt += 1
        self.setModel(model)
        return cnt

    def selectionChanged(self, selected, unselected):

        indexes = selected.indexes()
        if indexes:
            index = indexes[0]
            rowdata = index.internalPointer().rowdata
            emu = rowdata[-2]
            path = rowdata[-1]
            self.pathSelected.emit(emu, path)


class VivSymbolikFuncPane(e_q_memory.EnviNavMixin, vq_save.SaveableWidget, QWidget):

    viewidx = itertools.count()

    def __init__(self, vw, parent=None):
        self.vw = vw
        self.fva = None
        self.vwgui = vw.getVivGui()

        self.curemu = None
        self.cureffects = None

        self.symctx = viv_sym_analysis.getSymbolikAnalysisContext(vw, consolve=True)
        self.symexpr = viv_sym_expression.SymbolikExpressionParser(defwidth=vw.psize)

        if self.symctx is None:
            raise Exception('No Symboliks For: %s (yet)' % vw.getMeta('Architecture'))

        QWidget.__init__(self, parent=parent)
        e_q_memory.EnviNavMixin.__init__(self)
        self.setEnviNavName('Symboliks%d' % next(self.viewidx))

        self.exprtext = QLineEdit(parent=self)
        self.constraintext = QLineEdit(parent=self)
        self.alleffs = QCheckBox('all effects', parent=self)
        self.alleffs.stateChanged.connect(self.rendSymbolikPath)

        self.loop_count = QSpinBox(parent=self)
        looplabel = QLabel("Max Loops:", parent=self)

        self.pathview = VivSymbolikPathsView(vw, parent=self)
        self.pathview.setFixedHeight(100)
        self.memcanvas = e_q_memcanvas.VQMemoryCanvas(vw, syms=vw, parent=self)

        self.pathview.pathSelected.connect(self.symPathSelected)
        self.exprtext.returnPressed.connect(self.renderSymbolikPaths)
        self.constraintext.returnPressed.connect(self.renderSymbolikPaths)

        fvalabel = QLabel("Function VA:", parent=self)
        inccblabel = QLabel("Must Include VA:", parent=self)
        navbox = HBox(fvalabel, self.exprtext, inccblabel, self.constraintext, looplabel, self.loop_count, self.alleffs)

        mainbox = VBox()
        mainbox.addLayout(navbox)
        mainbox.addWidget(self.pathview)
        mainbox.addWidget(self.memcanvas)
        self.setLayout(mainbox)
        self.updateWindowTitle()

    def updateWindowTitle(self):
        ename = self.getEnviNavName()
        expr = str(self.exprtext.text())
        self.setWindowTitle('%s: %s' % (ename, expr))

    def enviNavGoto(self, expr, sizeexpr=None):
        self.exprtext.setText(expr)
        self.renderSymbolikPaths()
        self.updateWindowTitle()

    def vqGetSaveState(self):
        return {'expr': str(self.exprtext.text())}

    def vqSetSaveState(self, state):
        self.exprtext.setText(state.get('expr', ''))
        self.renderSymbolikPaths()

    def renderSymbolikPaths(self):
        try:

            self.memcanvas.clearCanvas()
            exprtxt = str(self.exprtext.text())
            if not exprtxt:
                return

            exprparts = exprtxt.split(';')
            expr = exprparts[0]

            preeff = []
            precons = []
            for e in exprparts[1:]:
                s = self.symexpr.parseExpression(e)

                if isinstance(s, viv_sym_common.Constraint):
                    precons.append(s)
                    continue

                if isinstance(s, viv_sym_effects.SymbolikEffect):
                    preeff.append(s)
                    continue

                raise Exception('Unhandled Symbolik Expression: %s' % e)

            self.symctx.setSymPreEffects(preeff)
            self.symctx.setSymPreConstraints(precons)

            va = self.vw.parseExpression(expr)
            self.fva = self.vw.getFunction(va)
            if self.fva is None:
                raise Exception('QTSymboliks: Invalid Address: 0x%.8x' % va)

            # check the constraints
            # FIXME: add ability to page through more than just the first 100 paths.  requires 
            #   storing the codegraph and codepaths
            codepaths = None
            codegraph = self.symctx.getSymbolikGraph(self.fva)
            cexpr = str(self.constraintext.text())
            if cexpr:
                cva = self.vw.parseExpression(cexpr)
                ccb = self.vw.getCodeBlock(cva)

                if ccb is not None and ccb in self.vw.getFunctionBlocks(self.fva):
                    loopcnt = self.loop_count.value()
                    codepaths = viv_graph.getCodePathsThru(codegraph, ccb[0], loopcnt=loopcnt)
                    paths = self.symctx.getSymbolikPaths(self.fva, paths=codepaths, graph=codegraph, maxpath=100)

            if codepaths is None:
                loopcnt = self.loop_count.value()
                paths = self.symctx.walkSymbolikPaths(self.fva, maxpath=100, loopcnt=loopcnt)

            if not self.pathview.loadSymbolikPaths(paths):
                self.memcanvas.addText('No valid symbolik paths found for %s' % expr)

        except Exception as e:
            self.memcanvas.addText('ERROR: %s' % e)
            self.memcanvas.addText(traceback.format_exception(*sys.exc_info()))

    def addVivNames(self, path, symobj, ctx):
        emu, symctx = ctx

        width = emu.__width__ # FIXME factory thing?

        if isinstance(symobj, viv_sym_common.Const):
            loc = self.vw.getLocation(symobj.value)
            if loc and loc[2] == LOC_STRING:
                s = repr(self.vw.readMemory(loc[0], loc[1]))
                s = '"%s"' % s[1:-1]
                return viv_sym_common.Var(s, width)

            if emu.isLocalMemory(symobj):
                offset = emu.getLocalOffset(symobj)
                # ltype,lname = self.vw.getFunctionLocal( self.fva, offset )
                flocal = self.vw.getFunctionLocal(self.fva, offset)
                if flocal is None:
                    flocal = ('int', 'local%d' % abs(offset))

                ltype, lname = flocal
                symobj.ptrname = lname

                return symobj

            if loc and loc[2] == LOC_UNI:
                buf = self.vw.readMemory(loc[0], loc[1])
                return viv_sym_common.Var('L"%s"' % buf.decode('utf-16le', 'ignore'), width)

            name = self.vw.getName(symobj.value)
            if name is not None:
                symobj = viv_sym_common.Var(name, width)

        return symobj

    def symPathSelected(self, emu, effects):
        self.curemu = emu
        self.cureffects = effects
        try:
            self.rendSymbolikPath()
        except Exception as e:
            import sys
            sys.excepthook(*sys.exc_info())

    def rendSymbolikPath(self, *args, **kwargs):
        '''
        Render the events from the currently selected emu/events.
        NOTE: args/kwargs syntax to allow arbitrary slot use
        '''
        emu = self.curemu
        effects = self.cureffects

        alleff = self.alleffs.checkState() == QtCore.Qt.Checked

        self.memcanvas.clearCanvas()
        colormap = {}
        for effect in effects:

            colormap[effect.va] = 'yellow'

            effect.reduce(emu)

            if alleff:
                effect.walkTree(self.addVivNames, ctx=(emu, self.symctx))
                self.memcanvas.addVaText('0x%.8x: ' % effect.va, effect.va)
                effect.render(self.memcanvas, vw=self.vw)
                self.memcanvas.addText('\n')
                continue

            if effect.efftype in (EFFTYPE_CONSTRAIN, EFFTYPE_CALLFUNC):
                effect.walkTree(self.addVivNames, ctx=(emu, self.symctx))
                self.memcanvas.addVaText('0x%.8x: ' % effect.va, effect.va)
                effect.render(self.memcanvas, vw=self.vw)
                self.memcanvas.addText( '\n' )
                continue

            if effect.efftype == EFFTYPE_WRITEMEM:
                if not emu.isLocalMemory(effect.symaddr):
                    effect.walkTree(self.addVivNames, ctx=(emu, self.symctx))
                    self.memcanvas.addVaText('0x%.8x: ' % effect.va, effect.va)
                    effect.render(self.memcanvas, vw=self.vw)
                    self.memcanvas.addText('\n')
                continue

        vqtevent('viv:colormap', colormap)

        retsym = emu.getFunctionReturn()
        retsym = retsym.reduce()

        self.memcanvas.addText('RETURNS: %s\n' % retsym)



"""
The envi architecture module for the AMD 64 platform.
"""
import struct

import envi
import envi.exc as e_exc
from envi.const import *
import envi.archs.i386 as e_i386

from envi.archs.amd64.regs import *
from envi.archs.amd64.disasm import *

# NOTE: The REX prefixes don't end up with displayed names
# NOTE: the REX prefix must be the *last* non escape (0f) prefix

# EMU NOTES:
# In 64 bit mode, all 32 bit dest regs get 0 extended into the rest of the bits
# In 64 bit mode, all 8/16 bit accesses do NOT modify the upper bits
# In 64 bit mode, all near branches, and implicit RSP (push pop) use RIP even w/o REX
# In 64 bit mode, if mod/rm is mod=0 and r/m is 5, it's RIP relative IMM32

class Amd64Module(e_i386.i386Module):

    def __init__(self):
        envi.ArchitectureModule.__init__(self, "amd64")
        self._arch_dis = Amd64Disasm()

    def initRegGroups(self):
        envi.ArchitectureModule.initRegGroups(self)
        self._regGrps.update({'general': ['rax', 'rbx', 'rcx', 'rdx', 'rsi', 'rdi', 'rbp',
                                'rsp', 'rip', 'r8', 'r9', 'r10', 'r11', 'r12',
                                'r13', 'r14', 'r15']} )

    def archGetRegCtx(self):
        return Amd64RegisterContext()

    def getPointerSize(self):
        return 8

    def pointerString(self, va):
        return "0x%.8x" % va

    def getEmulator(self):
        return Amd64Emulator()

class MSx64Call(envi.CallingConvention):
    '''
    Has space for 4 homing parameters.
    TODO: this is a compiler option.  We should have another convention if
    this is not specified.
    '''
    arg_def = [(CC_REG, REG_RCX), (CC_REG, REG_RDX), (CC_REG, REG_R8),
                (CC_REG, REG_R9), (CC_STACK_INF, 8*4+8),]
    retaddr_def = (CC_STACK, 0)
    retval_def = (CC_REG, REG_RAX)
    flags = CC_CALLER_CLEANUP
    align = 8
    pad = 8*4

class SysVAmd64Call(envi.CallingConvention):
    '''
    Does not have shadow space like MSx64.
    '''
    arg_def = [(CC_REG, REG_RDI), (CC_REG, REG_RSI), (CC_REG, REG_RDX),
                (CC_REG, REG_RCX), (CC_REG, REG_R8), (CC_REG, REG_R9),
                    (CC_STACK_INF, 8),]
    retaddr_def = (CC_STACK, 0)
    retval_def = (CC_REG, REG_RAX)
    flags = CC_CALLEE_CLEANUP
    align = 8
    pad = 0

class SysVAmd64SystemCall(SysVAmd64Call):
    '''
    For system calls, R10 is used instead of RCX.
    '''
    arg_def = [(CC_REG, REG_RDI), (CC_REG, REG_RSI), (CC_REG, REG_RDX),
                (CC_REG, REG_R10), (CC_REG, REG_R8), (CC_REG, REG_R9),
                (CC_STACK_INF, 8),]

sysvamd64call = SysVAmd64Call()
sysvamd64systemcall = SysVAmd64SystemCall()
msx64call = MSx64Call()

class Amd64Emulator(Amd64RegisterContext, e_i386.IntelEmulator):

    flagidx = REG_EFLAGS
    accumreg = { 1:REG_AL, 2:REG_AX, 4:REG_EAX, 8:REG_RAX }

    def __init__(self):
        archmod = Amd64Module()
        e_i386.IntelEmulator.__init__(self, archmod=archmod)
        # The above sets up the intel reg context, so we smash over it
        Amd64RegisterContext.__init__(self)
        # For the format calls in reading memory
        self.imem_psize = 8

        self.addCallingConvention("sysvamd64call", sysvamd64call)
        self.addCallingConvention("sysvamd64systemcall", sysvamd64systemcall)
        self.addCallingConvention("msx64call", msx64call)

    def doPush(self, val, size=8):
        rsp = self.getRegister(REG_RSP)
        rsp -= size
        self.writeMemValue(rsp, val, size)
        self.setRegister(REG_RSP, rsp)

    def doPop(self, size=8):
        rsp = self.getRegister(REG_RSP)
        val = self.readMemValue(rsp, size)
        self.setRegister(REG_RSP, rsp+size)
        return val

    def i_aam(self, op):
        raise e_exc.UnsupportedInstruction(self, op)

    i_aas = i_aam

    def i_pinsrq(self, op):
        self.i_pinsrb(op, width=8)

    def i_stosd(self, op):
        # TODO: Once we fix the postfix handling, come back and mop this up
        if op.prefixes & PREFIX_REX_W:
            rax = self.getRegister(REG_RAX)
            rdi = self.getRegister(REG_RDI)
            self.writeMemory(rdi, struct.pack("<Q", rax))
            if self.getFlag(EFLAGS_DF):
                rdi -= 8
            else:
                rdi += 8
            self.setRegister(REG_RDI, rdi)
        else:
            e_i386.IntelEmulator.i_stosd(self, op)

    def i_pextrd_q(self, op):
        if op.prefixes & PREFIX_REX_W:
            self.i_pextrb(op, width=8)
        else:
            self.i_pextrb(op, width=4)

    def i_div(self, op):
        tsize = op.opers[0].tsize
        if tsize == 8:
            val = self.getOperValue(op, 0)
            if val == 0:
                raise e_exc.DivideByZero(self)

            rax = self.getRegister(REG_RAX)
            rdx = self.getRegister(REG_RDX)
            tot = (rdx << 64) + rax
            quot = int(tot // val)
            rem = tot % val

            if tot > (2**128)-1:
                mesg = '0x%.8x: division exception on %s' % (op.va, str(op))
                raise e_exc.DivideError(self, msg=mesg)

            self.setRegister(REG_RAX, quot)
            self.setRegister(REG_RDX, rem)
        else:
            e_i386.IntelEmulator.i_div(self, op)

    def i_idiv(self, op):
        tsize = op.opers[0].tsize
        if tsize == 8:
            val = self.twoRegCompound(REG_RDX, REG_RAX, 8)
            val = e_bits.signed(val, 16)
            d = self.getOperValue(op, 0)
            d = e_bits.signed(d, 8)
            if d == 0:
                raise envi.DivideByZero(self)
            sign = (val < 0 and d > 0) or (val > 0 and d < 0)
            q = (abs(val) // abs(d))
            r = (abs(val) % abs(d))
            if sign:
                q = -q
                r = -r

            self.setRegister(REG_RAX, q)
            self.setRegister(REG_RDX, r)
        else:
            e_i386.IntelEmulator.i_idiv(self, op)




import envi.archs.h8.emu as h8_emu
import envi.archs.h8.regs as h8_regs
import vivisect.impemu.emulator as v_i_emulator


class H8WorkspaceEmulator(v_i_emulator.WorkspaceEmulator, h8_emu.H8Emulator):
    __archemu__ = h8_emu.H8Emulator

    taintregs = [h8_regs.REG_ER0, h8_regs.REG_ER1, h8_regs.REG_ER2]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        h8_emu.H8Emulator.__init__(self)
        v_i_emulator.WorkspaceEmulator.__init__(self, vw, **kwargs)



import logging

import envi
import envi.exc as e_exc
import envi.archs.arm as e_arm
from envi.archs.arm.regs import *

import vivisect.exc as v_exc
import vivisect.impemu.emulator as v_i_emulator

import visgraph.pathcore as vg_path

logger = logging.getLogger(__name__)


class ArmWorkspaceEmulator(v_i_emulator.WorkspaceEmulator, e_arm.ArmEmulator):
    __archemu__ = e_arm.ArmEmulator

    taintregs = [x for x in range(13)]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        e_arm.ArmEmulator.__init__(self)
        v_i_emulator.WorkspaceEmulator.__init__(self, vw, **kwargs)
        self.setMemArchitecture(envi.ARCH_ARMV7)

        va = kwargs.get('va')
        if va:
            self._prep(va)

    def setThumbMode(self, thumb=1):
        e_arm.ArmEmulator.setThumbMode(self, thumb)

    def setArmMode(self, arm=1):
        e_arm.ArmEmulator.setArmMode(self, arm)

    def parseOpcode(self, va, arch=envi.ARCH_DEFAULT):
        '''
        Caching version.

        We can make an opcode *faster* with the workspace because of
        getByteDef etc... use it.

        Made for ARM, because envi.Emulator doesn't understand the Thumb flag
        '''
        if arch == envi.ARCH_DEFAULT:
            tmode = self.getFlag(PSR_T_bit)
            arch = (envi.ARCH_ARMV7, envi.ARCH_THUMB)[tmode]

        return self.vw.parseOpcode(va, arch=arch)

    def stepi(self):
        # NOTE: when we step, we *always* want to be stepping over calls
        # (and possibly import emulate them)
        starteip = self.getProgramCounter()

        # parse out an opcode
        tmode = self.getFlag(PSR_T_bit)
        # logger.debug("tmode: %x", tmode)
        op = self.parseOpcode(starteip | tmode)
        if self.emumon:
            self.emumon.prehook(self, op, starteip)

        # Execute the opcode
        self.executeOpcode(op)
        vg_path.getNodeProp(self.curpath, 'valist').append(starteip)

        endeip = self.getProgramCounter()

        if self.emumon:
            self.emumon.posthook(self, op, endeip)

        if not self.checkCall(starteip, endeip, op):
            self.checkBranches(starteip, endeip, op)

    def _prep(self, funcva, tmode=None):
        if tmode is not None:
            # we're forcing thumb or arm mode... update the flag
            self.setFlag(PSR_T_bit, tmode)
            logger.debug("funcva thumb==%d  (forced):  0x%x", tmode, funcva)

        elif funcva & 3:
            # if the va isn't 4-byte aligned, it's gotta be thumb
            self.setFlag(PSR_T_bit, 1)
            funcva &= -2
            logger.debug("funcva is THUMB(addr):  0x%x", funcva)

        else:
            loc = self.vw.getLocation(funcva)
            if loc is not None:
                # if we have a opcode location, use it's iflags to determine mode
                lva, lsz, lt, lti = loc
                if (lti & envi.ARCH_MASK) == envi.ARCH_THUMB:
                    self.setFlag(PSR_T_bit, 1)
                    logger.debug("funcva is THUMB(loc):  0x%x", funcva)
                else:
                    logger.debug("funcva is ARM(loc):  0x%x", funcva)

            else:
                # otherwise, let's use some heuristics to guess.
                #
                # basically, the heuristic is made up of:
                # * does the current location decode as ARM or THUMB?
                # * if both, check the mnemonic against common opcodes 
                #       for start of functions
                #
                # armthumb is a tracker/counter.  
                #   positive numbers prefer ARM
                #   negative numbers prefer THUMB

                armthumb = 0
                armop  = None
                thumbop = None
                try:
                    thumbop = self.parseOpcode(funcva | 1)
                    armthumb -= 1
                    if thumbop.mnem == 'push':
                        armthumb -= 5
                    elif thumbop.mnem == 'ldr':
                        armthumb -= 2
                    elif thumbop.mnem == 'b':
                        armthumb -= 2

                except InvalidInstruction as e:
                    logger.debug("  heuristics: decoding ARM: %r", e)


                try:
                    armop = self.parseOpcode(funcva)
                    armthumb += 1
                    if armop.mnem == 'push':
                        armthumb += 5
                    elif armop.mnem == 'ldr':
                        armthumb += 2

                    if armop.prefixes: # highly unlikely for a function to start with a conditional
                        armthumb -= 3

                except InvalidInstruction as e:
                    logger.debug("  heuristics: decoding THUMB: %r", e)


                if armop is None and thumbop is None:
                    # we didn't have a single push in either direction
                    logger.warning("TOTAL FAILURE TO DETERMINE THUMB MODE")
                    raise Exception("Neither architecture parsed the first opcode")

                elif armthumb < 0:
                    self.setFlag(PSR_T_bit, 1)
                    logger.debug("ArmWorkspaceEmulator: Heuristically Determined funcva is THUMB:  0x%x", funcva)

                else:
                    self.setFlag(PSR_T_bit, 0)
                    logger.debug("ArmWorkspaceEmulator: Heuristically Determined funcva is ARM:  0x%x", funcva)


        self.funcva = funcva
        return funcva


    def runFunction(self, funcva, stopva=None, maxhit=None, maxloop=None, tmode=None):
        """
        This is a utility function specific to WorkspaceEmulation (and impemu) that
        will emulate, but only inside the given function.  You may specify a stopva
        to return once that location is hit.
        """
        logger.debug('=== emu.runFunction(0x%x, stopva=%r, maxhit=%r, maxloop=%r, tmode=%r)', funcva, stopva, maxhit, maxloop, tmode)
        funcva = self._prep(funcva, tmode)


        # Let the current (should be base also) path know where we are starting
        vg_path.setNodeProp(self.curpath, 'bva', funcva)

        hits = {}
        todo = [(funcva, self.getEmuSnap(), self.path)]
        vw = self.vw  # Save a dereference many many times

        while len(todo):

            va, esnap, self.curpath = todo.pop()

            self.setEmuSnap(esnap)

            self.setProgramCounter(va)
            tmode = self.getFlag(PSR_T_bit)

            # Check if we are beyond our loop max...
            if maxloop is not None:
                lcount = vg_path.getPathLoopCount(self.curpath, 'bva', va)
                if lcount > maxloop:
                    continue

            while True:

                starteip = self.getProgramCounter()

                if not vw.isValidPointer(starteip):
                    break

                if starteip == stopva:
                    return

                # Check straight hit count...
                if maxhit is not None:
                    h = hits.get(starteip, 0)
                    h += 1
                    if h > maxhit:
                        break
                    hits[starteip] = h

                # If we ran out of path (branches that went
                # somewhere that we couldn't follow)?
                if self.curpath is None:
                    break

                try:
                    # FIXME unify with stepi code...
                    op = self.parseOpcode(starteip | tmode)

                    self.op = op
                    if self.emumon:
                        try:
                            self.emumon.prehook(self, op, starteip)
                        except v_exc.BadOpBytes as e:
                            logger.debug(str(e))
                            break
                        except v_exc.BadOutInstruction:
                            pass
                        except Exception as e:
                            logger.log(self._log_level, "funcva: 0x%x opva: 0x%x:  %r   (%r) (in emumon prehook: %r)", funcva, starteip, op, e, self.emumon)

                        if self.emustop:
                            return

                    # Execute the opcode
                    self.executeOpcode(op)
                    vg_path.getNodeProp(self.curpath, 'valist').append(starteip)

                    endeip = self.getProgramCounter()

                    if self.emumon:
                        try:
                            self.emumon.posthook(self, op, endeip)
                        except v_exc.BadOpBytes as e:
                            logger.debug(str(e))
                            break
                        except v_exc.BadOutInstruction:
                            pass
                        except Exception as e:
                            logger.log(self._log_level, "funcva: 0x%x opva: 0x%x:  %r   (%r) (in emumon posthook: %r)", funcva, starteip, op, e, self.emumon)

                        if self.emustop:
                            return

                    iscall = self.checkCall(starteip, endeip, op)
                    if self.emustop:
                        return

                    # If it wasn't a call, check for branches, if so, add them to
                    # the todo list and go around again...
                    if not iscall:
                        blist = self.checkBranches(starteip, endeip, op)
                        if len(blist):
                            # pc in the snap will be wrong, but over-ridden at restore
                            esnap = self.getEmuSnap()
                            for bva, bpath in blist:
                                todo.append((bva, esnap, bpath))
                            break

                    else:
                        # check if we've blx'd to a different thumb state.  if so,
                        # be sure to return to the original tmode before continuing emulation pass
                        newtmode = self.getFlag(PSR_T_bit)
                        if newtmode != tmode:
                            self.setFlag(PSR_T_bit, tmode)

                    # If we enounter a procedure exit, it doesn't
                    # matter what EIP is, we're done here.
                    if op.iflags & envi.IF_RET:
                        vg_path.setNodeProp(self.curpath, 'cleanret', True)
                        break

                    # TODO: hook things like error(...) when they have a param that indicates to
                    # exit. Might be a bit hairy since we'll possibly have to fix up codeblocks
                    # Make sure we can at least get past the first instruction in certain functions
                    if self.vw.isNoReturnVa(op.va) and op.va != funcva:
                        vg_path.setNodeProp(self.curpath, 'cleanret', False)
                        break

                except envi.BadOpcode:
                    break
                except envi.UnsupportedInstruction as e:
                    if self.strictops:
                        logger.debug('runFunction breaking after unsupported instruction: 0x%08x %s', e.op.va, e.op.mnem)
                        raise e
                    else:
                        logger.debug('runFunction continuing after unsupported instruction: 0x%08x %s', e.op.va, e.op.mnem)
                        self.setProgramCounter(e.op.va + e.op.size)
                except v_exc.BadOutInstruction:
                    break
                except Exception as e:
                    if self.emumon is not None and not isinstance(e, e_exc.BreakpointHit):
                        self.emumon.logAnomaly(self, starteip, str(e))
                    logger.debug('runFunction breaking after exception (fva: 0x%x): %s', funcva, e)
                    break # If we exc during execution, this branch is dead.

class ThumbWorkspaceEmulator(ArmWorkspaceEmulator):
    def __init__(self, vw, **kwargs):
        ArmWorkspaceEmulator.__init__(self, vw, **kwargs)
        self.setThumbMode()
        self.setMemArchitecture(envi.ARCH_THUMB)

    def runFunction(self, funcva, stopva=None, maxhit=None, maxloop=None, tmode=None):
        return ArmWorkspaceEmulator.runFunction(self, funcva, stopva, maxhit, maxloop, tmode=1)

class Thumb16WorkspaceEmulator(ArmWorkspaceEmulator):
    def __init__(self, vw, **kwargs):
        ArmWorkspaceEmulator.__init__(self, vw, **kwargs)
        self.setThumbMode()
        self.setMemArchitecture(envi.ARCH_THUMB16)

    def runFunction(self, funcva, stopva=None, maxhit=None, maxloop=None, tmode=None):
        return ArmWorkspaceEmulator.runFunction(self, funcva, stopva, maxhit, maxloop, tmode=1)
'''
st0len gratuitously from wikipedia:

    ARM[edit]
    The standard ARM calling convention allocates the 16 ARM registers as:
    r15 is the program counter.
    r14 is the link register. (The BL instruction, used in a subroutine call, stores the return address in this register).
    r13 is the stack pointer. (The Push/Pop instructions in "Thumb" operating mode use this register only).
    r12 is the Intra-Procedure-call scratch register.
    r4 to r11: used to hold local variables.
    r0 to r3: used to hold argument values passed to a subroutine, and also hold results returned from a subroutine.

    If the type of value returned is too large to fit in r0 to r3, or whose size cannot be determined statically at compile time, then the caller must allocate space for that value at run time, and pass a pointer to that space in r0.

    Subroutines must preserve the contents of r4 to r11 and the stack pointer. (Perhaps by saving them to the stack in the function prologue, then using them as scratch space, then restoring them from the stack in the function epilogue). In particular, subroutines that call other subroutines *must* save the return address in the link register r14 to the stack before calling those other subroutines. However, such subroutines do not need to return that value to r14-they merely need to load that value into r15, the program counter, to return.

    The ARM stack is full-descending.[3]

    This calling convention causes a "typical" ARM subroutine to
    * In the prolog, push r4 to r11 to the stack, and push the return address in r14, to the stack. (This can be done with a single STM instruction).
    * copy any passed arguments (in r0 to r3) to the local scratch registers (r4 to r11).
    * allocate other local variables to the remaining local scratch registers (r4 to r11).
    * do calculations and call other subroutines as necessary using BL, assuming r0 to r3, r12 and r14 will not be preserved.
    * put the result in r0
    * In the epilog, pull r4 to r11 from the stack, and pulls the return address to the program counter r15. (This can be done with a single LDM instruction).
'''



"""
Solaris Platform Module (Incomplete)
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import os
import struct
import array

# Control codes (long values) for messages written to ctl and lwpctl files.
PCNULL   = 0  # null request, advance to next message */
PCSTOP   = 1  # direct process or lwp to stop and wait for stop */
PCDSTOP  = 2  # direct process or lwp to stop */
PCWSTOP  = 3  # wait for process or lwp to stop, no timeout */
PCTWSTOP = 4  # wait for stop, with long millisecond timeout arg */
PCRUN    = 5  # make process/lwp runnable, w/ long flags argument */
PCCSIG   = 6  # clear current signal from lwp */
PCCFAULT = 7  # clear current fault from lwp */
PCSSIG   = 8  # set current signal from siginfo_t argument */
PCKILL   = 9  # post a signal to process/lwp, long argument */
PCUNKILL = 10 # delete a pending signal from process/lwp, long arg */
PCSHOLD  = 11 # set lwp signal mask from sigset_t argument */
PCSTRACE = 12 # set traced signal set from sigset_t argument */
PCSFAULT = 13 # set traced fault set from fltset_t argument */
PCSENTRY = 14 # set traced syscall entry set from sysset_t arg */
PCSEXIT  = 15 # set traced syscall exit set from sysset_t arg */
PCSET    = 16 # set modes from long argument */
PCUNSET  = 17 # unset modes from long argument */
PCSREG   = 18 # set lwp general registers from prgregset_t arg */
PCSFPREG = 19 # set lwp floating-point registers from prfpregset_t */
PCSXREG  = 20 # set lwp extra registers from prxregset_t arg */
PCNICE   = 21 # set nice priority from long argument */
PCSVADDR = 22 # set %pc virtual address from long argument */
PCWATCH  = 23 # set/unset watched memory area from prwatch_t arg */
PCAGENT  = 24 # create agent lwp with regs from prgregset_t arg */
PCREAD   = 25 # read from the address space via priovec_t arg */
PCWRITE  = 26 # write to the address space via priovec_t arg */
PCSCRED  = 27 # set process credentials from prcred_t argument */
PCSASRS  = 28 # set ancillary state registers from asrset_t arg */
PCSPRIV  = 29 # set process privileges from prpriv_t argument */
PCSZONE  = 30 # set zoneid from zoneid_t argument */
PCSCREDX = 31 # as PCSCRED but with supplemental groups */

# PCRUN long operand flags.
PRCSIG   = 0x01# clear current signal, if any */
PRCFAULT = 0x02# clear current fault, if any */
PRSTEP   = 0x04# direct the lwp to single-step */
PRSABORT = 0x08# abort syscall, if in syscall */
PRSTOP   = 0x10# set directed stop request */

# Status flags
PR_STOPPED  = 0x00000001# lwp is stopped */
PR_ISTOP    = 0x00000002# lwp is stopped on an event of interest */
PR_DSTOP    = 0x00000004# lwp has a stop directive in effect */
PR_STEP     = 0x00000008# lwp has a single-step directive in effect */
PR_ASLEEP   = 0x00000010# lwp is sleeping in a system call */
PR_PCINVAL  = 0x00000020# contents of pr_instr undefined */
PR_ASLWP    = 0x00000040# obsolete flag; never set */
PR_AGENT    = 0x00000080# this lwp is the /proc agent lwp */
PR_DETACH   = 0x00000100# this is a detached lwp */
PR_DAEMON   = 0x00000200# this is a daemon lwp */
# The following flags apply to the process, not to an individual lwp */
PR_ISSYS    = 0x00001000# this is a system process */
PR_VFORKP   = 0x00002000# process is the parent of a vfork()d child */
PR_ORPHAN   = 0x00004000# process's process group is orphaned */
# The following process flags are modes settable by PCSET/PCUNSET */
PR_FORK     = 0x00100000# inherit-on-fork is in effect */
PR_RLC      = 0x00200000# run-on-last-close is in effect */
PR_KLC      = 0x00400000# kill-on-last-close is in effect */
PR_ASYNC    = 0x00800000# asynchronous-stop is in effect */
PR_MSACCT   = 0x01000000# micro-state usage accounting is in effect */
PR_BPTADJ   = 0x02000000# breakpoint trap pc adjustment is in effect */
PR_PTRACE   = 0x04000000# ptrace-compatibility mode is in effect */
PR_MSFORK   = 0x08000000# micro-state accounting inherited on fork */
PR_IDLE     = 0x10000000# lwp is a cpu's idle thread */


# Permissions...
MA_READ    = 0x04# readable by the traced process */
MA_WRITE   = 0x02# writable by the traced process */
MA_EXEC    = 0x01# executable by the traced process */
MA_SHARED  = 0x08# changes are shared by mapped object */
MA_ANON    = 0x40# anonymous memory (e.g. /dev/zero) */
MA_ISM     = 0x80# intimate shared mem (shared MMU resources) */
MA_NORESERVE = 0x100# mapped with MAP_NORESERVE */
MA_SHM     = 0x200# System V shared memory */
MA_RESERVED1 = 0x400# reserved for future use */

class SolarisMixin:

    def initMixin(self):
        #import sunprocfs
        self.ctl = None

    def platformGetRegs(self):
        pid = self.getPid()

    #def platformGetThreads(self):
        #ret = []
        #for name in os.listdir("/proc/%d/lwp" % self.pid):
            #ret.append(int(name))
        #return ret

    def platformAttach(self, pid):
        # TODO: uck. make a context handler pls
        self.ctl = open("/proc/%d/ctl" % pid, "ab")
        self.ctl.write(struct.pack("<L", PRSTOP))

    def platformContinue(self):
        """
        Tell the process to continue running
        """
        self.writeCtl(struct.pack("<LL", PCRUN, 0))

    def platformWait(self):
        """
        wait for the process to do someting "interesting"
        """
        self.writeCtl(struct.pack("<L", PCWSTOP))
        with open("/proc/%d/psinfo" % self.pid, "rb") as f:
            return f.read()

    def writeCtl(self, bytes):
        os.write(self.ctl.fileno(), bytes)

    def platformDetach(self):
        self.ctl.close()
        self.ctl = None

class SolarisIntelMixin:
    """
    Handle register formats for the intel solaris stuff
    """
    def getRegisterFormat(self):
        return ""

    def getRegisterNames(self):
        return []

    def platformReadMemory(self, addr, size):
        a = array.array('c',"\x00" * size)
        baddr, blen = a.buffer_info()
        priovec = struct.pack("<4L",PCREAD, baddr, blen, addr)
        self.writeCtl(priovec)
        return a.tostring()

    def platformWriteMemory(self, addr, bytes):
        a = array.array('c',bytes)
        baddr,blen = a.buffer_info()
        priovec = struct.pack("<LLLL", PCWRITE, baddr, blen, addr)
        self.writeCtl(priovec)

    def platformGetMaps(self):
        ret = []
        pid = self.getPid()
        with open("/proc/%d/map" % pid, "rb") as f:
            mapdata = f.read()
            while mapdata:
                addr,size = struct.unpack("<LL", mapdata[:8])
                perms, = struct.unpack("<L", mapdata[80:84])
                perms = perms & 0x7
                ret.append((addr,size, perms, ""))
                mapdata = mapdata[96:]
            return ret





'''
The vdb command / GUI extensions from vivisect!
'''
import logging
import traceback

import vdb
import vtrace
import vdb.qt.main as vdb_qt_main

import envi.cli as e_cli
import vivisect.colormap as viv_color

from envi.threads import firethread
from vqt.main import idlethread, idlethreadsync


logger = logging.getLogger(__name__)


class VivNotif(vtrace.Notifier):

    def __init__(self, vivgui):
        self.vivgui = vivgui

    @idlethreadsync
    def notify(self, event, trace):
        try:

            if event == vtrace.NOTIFY_BREAK and not trace.shouldRunAgain():

                pc = trace.getProgramCounter()
                if self.vivgui.vw.isValidPointer(pc):
                    self.vivgui.vivNavSignal.emit('0x%.8x' % pc)

                # If there are stalker hits, lets make a color map for them...
                shits = trace.getMeta('StalkerHits', None)
                if shits is not None:
                    self.vivgui.vw.vprint('vdb ext found stalker hits!')
                    cmap = viv_color.VivColorMap(self.vivgui.vw)
                    for hit in shits:
                        try:
                            cmap.colorBlock(hit, 'yellow')
                        except Exception as e:
                            self.vivgui.vw.vprint('WARNING: stalker color map: %s' % e)

                    #self.vivgui.setColorMap(cmap.getColorDict())
                    #cmap.saveAs('stalker')
                    cmap.setGuiMap()

        except Exception as e:
            logger.error(traceback.format_exc())


@firethread
def doLoad(db, trace, base):
    db.vw.vprint('Loading from memory 0x%.8x...' % base)
    db.vw.loadFromMemory(trace, base)
    db.vw.vprint('...loaded.')
    db.vw.analyze()


def vivimport(db, line):
    '''
    Import either a library or memory map from the target
    process into the current vivisect workspace.
    '''
    vw = db.vw
    trace = db.getTrace()

    argv = e_cli.splitargs(line)
    if len(argv) != 1:
        return db.do_help("vivimport")

    try:
        base = db.parseExpression(line)
    except Exception:
        db.vprint("Invalid Address Expression: %s" % argv[0])
        return

    # Init Arch/Format if they are not...
    vw.initMeta('Architecture', trace.getMeta('Architecture'))
    vw.initMeta('Format', trace.getMeta('Format'))

    doLoad(db, trace, base)


def extendVdb(db, vivgui):
    db.vw = vivgui.vw  # Store a ref to the workspace...
    db.vivgui = vivgui  # Also a reference to the GUI
    db.registerNotifier(vtrace.NOTIFY_ALL, VivNotif(vivgui))
    db.registerCmdExtension(vivimport)


@firethread
def fireattach(trace, pid):
    trace.attach(pid)


@idlethread
def runVdb(vivgui, pid=None, trace=None):
    try:
        db = vdb.Vdb(trace=trace)
        extendVdb(db, vivgui)
        vgui = vdb_qt_main.VdbWindow(db)
        vgui.show()
        if pid is not None:
            fireattach(db.trace, pid)
    except Exception as e:
        vivgui.vw.vprint('Error Running VDB: %s' % e)



import sys
import errno


def waitForTest():
    sys.stdout.write('testwait\n')
    sys.stdout.flush()
    while True:
        line = safeReadline().strip()
        if line == 'testmod':
            break


def safeReadline():
    while True:
        try:  # Crazy loop for freebsd readline failure
            r = sys.stdin.readline()
            break
        except IOError as e:
            if e.errno == errno.EINTR:
                continue
            raise
    return r


def exitTest():
    sys.exit(33)


def main():
    waitForTest()
    return 33


if __name__ == '__main__':
    sys.exit(main())
